package core;

import java.io.Serializable;


/**
 * Attributes
 * @author Varkalas D`Lonovan
 */
public class Attributes implements Serializable {
	/**
	 * Class Variables
	 */
	private static final long serialVersionUID = -8154190247591257424L;
	private int level;
	private int might;
	private int endurance;
	private int quickness;
	private int acumen;
	private int maximumHitPoints;
	private int maximumManaPoints;
	private int currentHitPoints;
	private int currentManaPoints;
	private int movementRate;


	/**
	 * Constructor
	 */
	public Attributes() {
		level = 0;
		might = 0;
		endurance = 0;
		quickness = 0;
		acumen = 0;
		maximumHitPoints = 0;
		maximumManaPoints = 0;
		currentHitPoints = 0;
		currentManaPoints = 0;
		movementRate = 0;
	}


	/**
	 * @return Returns the level.
	 */
	public int getLevel() {
		return level;
	}


	/**
	 * @param level The level to set.
	 */
	public void setLevel(int level) {
		this.level = level;
	}


	/**
	 * @return Returns the acumen.
	 */
	public int getAcumen() {
		return acumen;
	}


	/**
	 * @param acumen The acumen to set.
	 */
	public void setAcumen(int acumen) {
		this.acumen = acumen;
	}


	/**
	 * @return Returns the endurance.
	 */
	public int getEndurance() {
		return endurance;
	}


	/**
	 * @param endurance The endurance to set.
	 */
	public void setEndurance(int endurance) {
		this.endurance = endurance;
	}


	/**
	 * @return Returns the might.
	 */
	public int getMight() {
		return might;
	}


	/**
	 * @param might The might to set.
	 */
	public void setMight(int might) {
		this.might = might;
	}


	/**
	 * @return Returns the quickness.
	 */
	public int getQuickness() {
		return quickness;
	}


	/**
	 * @param quickness The quickness to set.
	 */
	public void setQuickness(int quickness) {
		this.quickness = quickness;
	}


	/**
	 * @return Returns the currentHitPoints.
	 */
	public int getCurrentHitPoints() {
		return currentHitPoints;
	}


	/**
	 * @param currentHitPoints The currentHitPoints to set.
	 */
	public void setCurrentHitPoints(int currentHitPoints) {
		if (currentHitPoints >= maximumHitPoints) {
			currentHitPoints = maximumHitPoints;
		} else {
			this.currentHitPoints = currentHitPoints;
		}
	}


	/**
	 * @return Returns the currentManaPoints.
	 */
	public int getCurrentManaPoints() {
		return currentManaPoints;
	}


	/**
	 * @param currentManaPoints The currentManaPoints to set.
	 */
	public void setCurrentManaPoints(int currentManaPoints) {
		if (currentManaPoints >= maximumManaPoints) {
			currentManaPoints = maximumManaPoints;
		} else {
			this.currentManaPoints = currentManaPoints;
		}
	}


	/**
	 * calculateMaximumHitandManaPoints
	 * used when leveling up, putting on/removing equipment, and training
	 * @param profession The profession of the character.
	 */
	public void calculateMaximumHitandManaPoints(int profession) {
		if (profession == Profession.WARRIOR) {
			maximumHitPoints = level * 12;
			maximumManaPoints = 0;
		} else if (profession == Profession.PALADIN || profession == Profession.RANGER) {
			maximumHitPoints = level * 10;
			maximumManaPoints = level * 4;
		} else if (profession == Profession.CLERIC) {
			maximumHitPoints = level * 8;
			maximumManaPoints = level * 8;
		} else if (profession == Profession.THIEF) {
			maximumHitPoints = level * 6;
			maximumManaPoints = 0;
		} else if (profession == Profession.SORCERER || profession == Profession.WARRIOR) {
			maximumHitPoints = level * 4;
			maximumManaPoints = level * 12;
		}
		maximumHitPoints += level * (might * .1) * (endurance * .5);
		maximumManaPoints += level * (acumen * .5);
	}


	/**
	 * @return Returns the movementRate.
	 */
	public int getMovementRate() {
		return movementRate;
	}


	/**
	 * @param movementRate The movementRate to set.
	 */
	public void setMovementRate(int movementRate) {
		this.movementRate = movementRate;
	}
}