package core;


/**
 * Imports
 */
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;

import javax.imageio.ImageIO;


/**
 * Character
 * @author Varkalas D`Lonovan
 */
public class GameEntity implements Serializable {
	/**
	 * Class Variables
	 */
	private static final long serialVersionUID = 1031140693164440182L;
    private String name;
    private Profession profession;
    private Attributes attributes;
    private Skills skills;
    private Equipment equipment;
    private int[] quickKeys;
    private World world;
    private Area currentArea;
    private Area previousArea;
    private Location currentLocation;
    private Location previousLocation;
    private BufferedImage sprite;
    private int type;
    private boolean selected;
    private int moving;
    private float movement;
    private boolean attacking;
    private boolean beingAttacked;
	public static final int STOPPED = 0;
	public static final int NORTH = 1;
	public static final int SOUTH = 2;
	public static final int WEST = 3;
	public static final int EAST = 4;
	public static final int PLAYER = 5;
	public static final int CREATURE_HOSTILE_GROUP = 6;
	public static final int CREATURE_HOSTILE_SOLO = 7;
	public static final int CREATURE_NEUTRAL_GROUP = 8;
	public static final int CREATURE_NEUTRAL_SOLO = 9;
	public static final int DAMAGE = 10;
	public static final int MAGIC = 11;


    /**
     * Constructor
     */
    public GameEntity(World world) {
    	name = "";
    	profession = new Profession();
    	attributes = new Attributes();
        skills = new Skills();
        equipment = new Equipment();
        quickKeys = new int[10];
        this.world = world;
        currentArea = world.getArea("TownTest1");
        previousArea = world.getArea("TownTest1");
        currentLocation = new Location();
        previousLocation = new Location();
        sprite = null;
        type = PLAYER;
        selected = false;
        moving = STOPPED;
        movement = 0;
        attacking = false;
        beingAttacked = false;
    }


	/**
     * newPlayer
     * @param name The name of the new character.
     * @param profession The profession of the new character.
     * @param type The type of character.
     * @param movementRate The speed of the character.
     */
    public void newPlayer(String name, String profession, int type, int movementRate) {
    	this.name = name;
    	this.profession.setName(profession);
    	try {
    		sprite = ImageIO.read(new File("Images/Sprites/Professions/" + profession + ".bmp"));
    	} catch (IOException exception) {
    		System.err.println("[GameEntity.java]  Could not read " + profession + ".bmp.");
    	}
    	this.type = type;
    	attributes.setMovementRate(movementRate);
    }


    /**
	 * defend
	 * @param damageTaken The damage taken by the entity.
	 */
	public void defend(int damageType, int damageTaken) {
		// damageTaken = return from attack
		// cycle through all equipment, get armor, get physical resistance
	}


	/**
	 * attack
	 * @param damageType Either physical or mental damage.
	 * @param defender The defending individual.
	 * @return Returns the damage that gets generated.
	 */
	public int attack(int damageType, GameEntity defender) {
		System.out.println("Player: " + attributes.getCurrentHitPoints());
		System.out.println("Enemy: " + defender.getAttributes().getCurrentHitPoints());
		if (damageType == DAMAGE) {
			double chanceMiss = attributes.getQuickness() - defender.getAttributes().getQuickness();
			if (chanceMiss < 0) {
				// attacker hit (small chance)
				if (RandomChance.didChanceSucceed(- 1 / chanceMiss)) {
					System.out.println(name + " hit.");
				// attacker misses (great chance)
				} else {
					System.out.println(name + " missed.");
				}
			} else if (chanceMiss >= 0){
				// attacker misses (small chance)
				if (RandomChance.didChanceSucceed(chanceMiss)) {
					System.out.println(name + " missed.");
				// attacker hit (great chance)
				} else {
					System.out.println(name + " hit.");
				}
			}
		} else if (damageType == MAGIC) {
			// TODO
		}

		// miss
		return -1;
	}


	/**
	 * adjacentToPlayer
	 * @return Returns whether the game entity is adjacent to any other game entity.
	 */
	public boolean adjacentToPlayer() {
		if (this.collisionDetection(NORTH) == 2 || this.collisionDetection(SOUTH) == 2
		 || this.collisionDetection(WEST) == 2 || this.collisionDetection(EAST) == 2) {
			return true;
		}

		return false;
	}


	/**
	 * move
	 * @param direction The direction he wishes to move in.
	 * @return Returns whether the character moved into another area or not.
	 */
	public boolean move(int direction) {
		int x = currentLocation.getX();
		int y = currentLocation.getY();

		if (type == PLAYER) {
			if (direction == NORTH) {
				currentLocation.setY(y + attributes.getMovementRate());
				if (currentLocation.getY() >= 256) {
					currentLocation.setY(-224);
	    			currentArea = world.getArea(currentArea.getAbove());
	    			return true;
	    		}
			} else if (direction == SOUTH) {
				currentLocation.setY(y - attributes.getMovementRate());
				if (currentLocation.getY() <= -256) {
					currentLocation.setY(224);
					currentArea = world.getArea(currentArea.getBelow());
					return true;
				}
			} else if (direction == WEST) {
				currentLocation.setX(x + attributes.getMovementRate());
				if (currentLocation.getX() >= 352) {
					currentLocation.setX(-288);
	    			currentArea = world.getArea(currentArea.getLeft());
	    			return true;
	    		}
			} else if (direction == EAST) {
				currentLocation.setX(x - attributes.getMovementRate());
				if (currentLocation.getX() <= -320) {
					currentLocation.setX(320);
	    			currentArea = world.getArea(currentArea.getRight());
	    			return true;
	    		}
			}
		} else {
			int npcDirection = 0;

			// the directions are opposite because it's relative to the player
			if (direction == NORTH) {
				npcDirection = SOUTH;
			} else if (direction == SOUTH) {
				npcDirection = NORTH;
			} else if (direction == WEST) {
				npcDirection = EAST;
			} else if (direction == EAST) {
				npcDirection = WEST;
			}

			if (npcDirection == NORTH) {
				currentLocation.setY(y + attributes.getMovementRate());
				if (currentLocation.getY() >= 480) {
					currentLocation.setY(0);
	    			currentArea = world.getArea(currentArea.getBelow());
	    			return true;
	    		}
			} else if (npcDirection == SOUTH) {
				currentLocation.setY(y - attributes.getMovementRate());
				if (currentLocation.getY() <= 0) {
					currentLocation.setY(480);
					currentArea = world.getArea(currentArea.getAbove());
					return true;
				}
			} else if (npcDirection == WEST) {
				currentLocation.setX(x + attributes.getMovementRate());
				if (currentLocation.getX() >= 640) {
					currentLocation.setX(0);
	    			currentArea = world.getArea(currentArea.getRight());
	    			return true;
	    		}
			} else if (npcDirection == EAST) {
				currentLocation.setX(x - attributes.getMovementRate());
				if (currentLocation.getX() <= 0) {
					currentLocation.setX(640);
	    			currentArea = world.getArea(currentArea.getLeft());
	    			return true;
	    		}
			}
		}

		return false;
	}


	/**
	 * enteringNewArea
	 * @param enteringDirection The direction the person is facing/entering the area.
	 */
	public void enteringNewArea(int enteringDirection) {
		int maskCheckX = (320 - currentLocation.getX()) / 32;
		int maskCheckY = (224 - currentLocation.getY()) / 32;

		// set the new area
		currentArea = world.getArea(currentArea.getEntrancePoints()[maskCheckX][maskCheckY]);

		// set the proper coordinates
		for (int index1 = 0; index1 < 20; index1++) {
			for (int index2 = 0; index2 < 15; index2++) {
				if (!currentArea.getEntrancePoints()[index1][index2].equals("")) {
					if (enteringDirection == NORTH) {
						currentLocation.setX(320 - (index1 * 32));
						currentLocation.setY(224 - ((index2 - 1) * 32));
					} else if (enteringDirection == SOUTH) {
						currentLocation.setX(320 - (index1 * 32));
						currentLocation.setY(224 - ((index2 + 1) * 32));
					} else if (enteringDirection == WEST) {
						currentLocation.setX(320 - ((index1 - 1)* 32));
						currentLocation.setY(224 - (index2 * 32));
					} else if (enteringDirection == EAST) {
						currentLocation.setX(320 - ((index1 + 1) * 32));
						currentLocation.setY(224 - (index2 * 32));
					}
				}
			}
		}
	}


	/**
	 * checkSameLocation
	 */
	public void checkSameLocation() {
		int counter = 0;

		// checks if some creature tried to get in the player's way
		if (type == PLAYER) {
			for (int index = 0; index < world.getNPCs().size(); index++) {
				GameEntity gameEntity = (GameEntity) world.getNPCs().get(index);
				if (320 - currentLocation.getX() == gameEntity.getCurrentLocation().getX() &&
					224 - currentLocation.getY() == gameEntity.getCurrentLocation().getY()) {
					gameEntity.setCurrentLocation(gameEntity.getPreviousLocation().getX(),
												  gameEntity.getPreviousLocation().getY());
					gameEntity.setCurrentArea(gameEntity.getPreviousArea());
				}
			}

		// checks if more than one creature are in the same location
		} else {
			for (int index = 0; index < world.getNPCs().size(); index++) {
				GameEntity gameEntity = (GameEntity) world.getNPCs().get(index);
				if (currentLocation.getX() == gameEntity.getCurrentLocation().getX() &&
					currentLocation.getY() == gameEntity.getCurrentLocation().getY()) {
					counter++;
				}
				if (counter > 1) {
					setCurrentLocation(getPreviousLocation().getX(), getPreviousLocation().getY());
					setCurrentArea(getPreviousArea());
					counter = 0;
				}
			}
		}
	}


	/**
     * collisionDetection
     * @param direction The direction the player wants to move.
     * @return 0 nothing, 1 inanimate object, 2 npc, 3 entrance, 4 player 
     */
    public int collisionDetection(int direction) {
    	int gameEntityX = currentLocation.getX();
    	int gameEntityY = currentLocation.getY();
    	int maskCheckX = (320 - gameEntityX) / 32;
		int maskCheckY = (224 - gameEntityY) / 32;

		if (type != PLAYER) {
			maskCheckX = gameEntityX / 32;
			maskCheckY = gameEntityY / 32;
			if (maskCheckX >= 20) {
				maskCheckX -= 1;
			}
			if (maskCheckY >= 15) {
				maskCheckY -= 1;
			}
		}


		// NORTH
		if (direction == GameEntity.NORTH) {
    		// OBSTACLES and ENTRANCES
	    	if (gameEntityY <= 224) {
				if (maskCheckY <= 0) {
					if (world.getArea(currentArea.getAbove()).getMask()[maskCheckX][14] == 1) {
						return 1;
					} else if (world.getArea(currentArea.getAbove()).getMask()[maskCheckX][14] == 2) {
						return 3;
					}
				} else {
					if (currentArea.getMask()[maskCheckX][maskCheckY - 1] == 1) {
						return 1;
					} else if (currentArea.getMask()[maskCheckX][maskCheckY - 1] == 2) {
						return 3;
					}
				}
			}

	    	// ENTITIES
	    	// creature vs. player check
	    	if (type != PLAYER) {
	    		int playerX = world.getPlayer().getCurrentLocation().getX();
	    		int playerY = world.getPlayer().getCurrentLocation().getY();

	    		if (world.getPlayer().getCurrentArea() == currentArea) {
	    			if (currentLocation.getX() == 320 - playerX) {
	    				if (currentLocation.getY() - 32 == 224 - playerY) {
	    					return 4;
	    				}
	    			}
	    		} else if (world.getPlayer().getCurrentArea() == world.getArea(currentArea.getAbove())) {
	    			if (currentLocation.getX() == 320 - playerX) {
	    				if (currentLocation.getY() - 32 + 480 == 224 - playerY) {
	    					return 4;
	    				}
	    			}
	    		}
	    	}
	    	// creature/player vs. creature check
	    	for (int index = 0; index < world.getNPCs().size(); index++) {
	    		GameEntity gameEntity = (GameEntity) world.getNPCs().get(index);
	    		int x;
	    		int y;

	    		if (gameEntity.getCurrentArea() == currentArea) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX();
		    			y = 224 - gameEntity.getCurrentLocation().getY() - 32;
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX();
	    				y = gameEntity.getCurrentLocation().getY() + 32;
	    			}
	    			if (x == currentLocation.getX()) {
	    				if (y == currentLocation.getY()) {
	    					return 2;
	    				}
	    			}
	    		} else if (gameEntity.getCurrentArea() == world.getArea(currentArea.getAbove())) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX();
		    			y = 224 - gameEntity.getCurrentLocation().getY() - 480 - 32;
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX();
	    				y = gameEntity.getCurrentLocation().getY() - 480 + 32;
	    			}
	    			if (x == currentLocation.getX()) {
	    				if (y == currentLocation.getY()) {
	    					return 2;
	    				}
	    			}
	    		}
	    	}
	    // SOUTH
    	} else if (direction == GameEntity.SOUTH) {
    		// OBSTACLES and ENTRANCES
	    	if (gameEntityY >= -224) {
				if (maskCheckY >= 14) {
					if (world.getArea(currentArea.getBelow()).getMask()[maskCheckX][0] == 1) {
						return 1;
					} else if (world.getArea(currentArea.getBelow()).getMask()[maskCheckX][0] == 2) {
						return 3;
					}
				} else {
					if (currentArea.getMask()[maskCheckX][maskCheckY + 1] == 1) {
						return 1;
					} else if (currentArea.getMask()[maskCheckX][maskCheckY + 1] == 2) {
						return 3;
					}
				}
			}
	    	// ENTITIES
	    	// creature vs. player check
	    	if (type != PLAYER) {
	    		int playerX = world.getPlayer().getCurrentLocation().getX();
	    		int playerY = world.getPlayer().getCurrentLocation().getY();

	    		if (world.getPlayer().getCurrentArea() == currentArea) {
	    			if (currentLocation.getX() == 320 - playerX) {
	    				if (currentLocation.getY() + 32 == 224 - playerY) {
	    					return 4;
	    				}
	    			}
	    		} else if (world.getPlayer().getCurrentArea() == world.getArea(currentArea.getBelow())) {
	    			if (currentLocation.getX() == 320 - playerX) {
	    				if (currentLocation.getY() + 32 - 480 == 224 - playerY) {
	    					return 4;
	    				}
	    			}
	    		}
	    	}
	    	// creature/player vs. creature check
	    	for (int index = 0; index < world.getNPCs().size(); index++) {
	    		GameEntity gameEntity = (GameEntity) world.getNPCs().get(index);
	    		int x;
	    		int y;

	    		if (gameEntity.getCurrentArea() == currentArea) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX();
		    			y = 224 - gameEntity.getCurrentLocation().getY() + 32;
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX();
	    				y = gameEntity.getCurrentLocation().getY() - 32;
	    			}
	    			if (x == currentLocation.getX()) {
	    				if (y == currentLocation.getY()) {
	    					return 2;
	    				}
	    			}
	    		} else if (gameEntity.getCurrentArea() == world.getArea(currentArea.getBelow())) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX();
		    			y = 224 - gameEntity.getCurrentLocation().getY() + 480 + 32;
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX();
	    				y = gameEntity.getCurrentLocation().getY() + 480 - 32;
	    			}
    				if (x == currentLocation.getX()) {
    					if (y == currentLocation.getY()) {
    						return 2;
    					}
    				}
	    		}
	    	}
	    // WEST
    	} else if (direction == GameEntity.WEST) {
    		// OBSTACLES and ENTRANCES
    		if (gameEntityX <= 320) {
				if (maskCheckX <= 0) {
					if (world.getArea(currentArea.getLeft()).getMask()[19][maskCheckY] == 1) {
						return 1;
					} else if (world.getArea(currentArea.getLeft()).getMask()[19][maskCheckY] == 2) {
						return 3;
					}
				} else {
    				if (currentArea.getMask()[maskCheckX - 1][maskCheckY] == 1) {
    					return 1;
    				} else if (currentArea.getMask()[maskCheckX - 1][maskCheckY] == 2) {
    					return 3;
    				}
				}
    		}

    		// ENTITIES
	    	// creature vs. player check
    		if (type != PLAYER) {
	    		int playerX = world.getPlayer().getCurrentLocation().getX();
	    		int playerY = world.getPlayer().getCurrentLocation().getY();

	    		if (world.getPlayer().getCurrentArea() == currentArea) {
	    			if (currentLocation.getX() - 32 == 320 - playerX) {
	    				if (currentLocation.getY() == 224 - playerY) {
	    					return 4;
	    				}
	    			}
	    		} else if (world.getPlayer().getCurrentArea() == world.getArea(currentArea.getLeft())) {
	    			if (currentLocation.getX() - 32 + 640 == 320 - playerX) {
	    				if (currentLocation.getY() == 224 - playerY) {
	    					return 4;
	    				}
	    			}
	    		}
	    	}
    		// creature/player vs. creature check
	    	for (int index = 0; index < world.getNPCs().size(); index++) {
	    		GameEntity gameEntity = (GameEntity) world.getNPCs().get(index);
	    		int x;
	    		int y;

	    		if (gameEntity.getCurrentArea() == currentArea) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX() - 32;
		    			y = 224 - gameEntity.getCurrentLocation().getY();
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX() + 32;
	    				y = gameEntity.getCurrentLocation().getY();
	    			}
	    			if (x == currentLocation.getX()) {
	    				if (y == currentLocation.getY()) {
	    					return 2;
	    				}
	    			}
	    		} else if (gameEntity.getCurrentArea() == world.getArea(currentArea.getLeft())) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX() + 640 - 32;
		    			y = 224 - gameEntity.getCurrentLocation().getY();
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX() + 640 + 32;
	    				y = gameEntity.getCurrentLocation().getY();
	    			}
	    			if (x == currentLocation.getX()) {
	    				if (y == currentLocation.getY()) {
	    					return 2;
	    				}
	    			}
	    		}
	    	}
	    // EAST
    	} else if (direction == GameEntity.EAST) {
    		// OBSTACLES and ENTRANCES
    		if (gameEntityX >= -320) {
				if (maskCheckX >= 19) {
					if (world.getArea(currentArea.getRight()).getMask()[0][maskCheckY] == 1) {
						return 1;
					} else if (world.getArea(currentArea.getRight()).getMask()[0][maskCheckY] == 2) {
						return 3;
					}
				} else {
    				if (currentArea.getMask()[maskCheckX + 1][maskCheckY] == 1) {
    					return 1;
    				} else if (currentArea.getMask()[maskCheckX + 1][maskCheckY] == 2) {
    					return 3;
    				}
				}
    		}

    		// ENTITIES
	    	// creature vs. player check
    		int playerX = world.getPlayer().getCurrentLocation().getX();
    		int playerY = world.getPlayer().getCurrentLocation().getY();

    		if (world.getPlayer().getCurrentArea() == currentArea) {
    			if (currentLocation.getX() + 32 == 320 - playerX) {
    				if (currentLocation.getY() == 224 - playerY) {
    					return 4;
    				}
    			}
    		} else if (world.getPlayer().getCurrentArea() == world.getArea(currentArea.getRight())) {
    			if (currentLocation.getX() + 32 - 640 == 320 - playerX) {
    				if (currentLocation.getY() == 224 - playerY) {
    					return 4;
    				}
    			}
    		}
    		// creature/player vs. creature check
	    	for (int index = 0; index < world.getNPCs().size(); index++) {
	    		GameEntity gameEntity = (GameEntity) world.getNPCs().get(index);
	    		int x;
	    		int y;
	    		if (gameEntity.getCurrentArea() == currentArea) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX() + 32;
		    			y = 224 - gameEntity.getCurrentLocation().getY();
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX() - 32;
	    				y = gameEntity.getCurrentLocation().getY();
	    			}
	    			if (x == currentLocation.getX()) {
	    				if (y == currentLocation.getY()) {
	    					return 2;
	    				}
	    			}
	    		} else if (gameEntity.getCurrentArea() == world.getArea(currentArea.getRight())) {
	    			if (type == PLAYER) {
		    			x = 320 - gameEntity.getCurrentLocation().getX() - 640 + 32;
		    			y = 224 - gameEntity.getCurrentLocation().getY();
	    			} else {
	    				x = gameEntity.getCurrentLocation().getX() - 640 - 32;
	    				y = gameEntity.getCurrentLocation().getY();
	    			}
	    			if (x == currentLocation.getX()) {
	    				if (y == currentLocation.getY()) {
	    					return 2;
	    				}
	    			}
	    		}
	    	}
    	}

    	return 0;
    }


	/**
	 * getName
	 * @return Returns the name of the player's character.
	 */
	public String getName() {
		return name;
	}


	/**
	 * setName
	 * @param name The name of the player's character.
	 */
	public void setName(String aName) {
		name = aName;
	}


	/**
	 * getProfession
	 * @return Returns the profession of the character.
	 */
	public Profession getProfession() {
		return profession;
	}


	/**
	 * getSkills
	 * @return Returns the skills.
	 */
	public Skills getSkills() {
		return skills;
	}


	/**
	 * getInventory
	 * @return Returns the character's inventory.
	 */
	public Equipment getEquipment() {
		return equipment;
	}


	/**
	 * getCurrentArea
	 * @return Returns the area the character is located in.
	 */
	public Area getCurrentArea() {
		return currentArea;
	}


	/**
	 * setCurrentArea
	 * @param anArea The place to be set as the entity's current area.
	 */
	public void setCurrentArea(Area anArea) {
		currentArea = anArea;
	}


	/**
	 * getCurrentLocation
	 * @return Returns the current location of the entity.
	 */
	public Location getCurrentLocation() {
		return currentLocation;
	}


	/**
	 * setCurrentLocation
	 * @param x The x position of the new location.
	 * @param y The y position of the new location.
	 */
	public void setCurrentLocation(int x, int y) {
		currentLocation.setX(x);
		currentLocation.setY(y);
	}


	/**
	 * getSprite
	 * @return Returns the sprite image of the character.
	 */
	public BufferedImage getSprite() {
		return sprite;
	}


	/**
	 * setSprite
	 * @param spriteName The name of the sprite of the NPC.
	 */
	public void setSprite(String spriteName) {
		try {
    		sprite = ImageIO.read(new File("Images/Sprites/NPCs/" + spriteName + ".bmp"));
    	} catch (IOException exception) {
    		System.err.println("[GameEntity.java]  Could not read " + spriteName + ".bmp.");
    	}
	}


	/**
	 * @return Returns the attributes.
	 */
	public Attributes getAttributes() {
		return attributes;
	}



	/**
	 * @param type The type to set.
	 */
	public void setType(String type) {
		if (type.equals("PLAYER")) {
			this.type = PLAYER;
		} else if (type.equals("CREATURE_HOSTILE_GROUP")) {
			this.type = CREATURE_HOSTILE_GROUP;
		} else if (type.equals("CREATURE_HOSTILE_SOLO")) {
			this.type = CREATURE_HOSTILE_SOLO;
		} else if (type.equals("CREATURE_NEUTRAL_GROUP")) {
			this.type = CREATURE_NEUTRAL_GROUP;
		} else if (type.equals("CREATURE_NEUTRAL_SOLO")) {
			this.type = CREATURE_NEUTRAL_SOLO;
		}
	}


	/**
	 * @return Returns the quickKeys.
	 */
	public int[] getQuickKeys() {
		return quickKeys;
	}


	/**
	 * @return Returns the selected.
	 */
	public boolean isSelected() {
		return selected;
	}


	/**
	 * @param selected The selected to set.
	 */
	public void setSelected(boolean selected) {
		this.selected = selected;
	}


	/**
	 * @return Returns the attacking.
	 */
	public boolean isAttacking() {
		return attacking;
	}


	/**
	 * @param attacking The attacking to set.
	 */
	public void setAttacking(boolean attacking) {
		this.attacking = attacking;
	}


	/**
	 * @return Returns the beingAttacked.
	 */
	public boolean isBeingAttacked() {
		return beingAttacked;
	}


	/**
	 * @param beingAttacked The beingAttacked to set.
	 */
	public void setBeingAttacked(boolean beingAttacked) {
		this.beingAttacked = beingAttacked;
	}


	/**
	 * @return Returns the moving.
	 */
	public int getMoving() {
		return moving;
	}


	/**
	 * @param moving The moving to set.
	 */
	public void setMoving(int moving) {
		this.moving = moving;
	}


	/**
	 * @return Returns the movement.
	 */
	public float getMovement() {
		return movement;
	}


	/**
	 * @param movement The movement to set.
	 */
	public void setMovement(float movement) {
		this.movement = movement;
	}


	/**
	 * @return Returns the previousLocation.
	 */
	public Location getPreviousLocation() {
		return previousLocation;
	}


	/**
	 * setPreviousLocation
	 * @param x The x position of the previous location.
	 * @param y The y position of the previous location.
	 */
	public void setPreviousLocation(int x, int y) {
		previousLocation.setX(x);
		previousLocation.setY(y);
	}


	/**
	 * @return Returns the previousArea.
	 */
	public Area getPreviousArea() {
		return previousArea;
	}


	/**
	 * @param previousArea The previousArea to set.
	 */
	public void setPreviousArea(Area previousArea) {
		this.previousArea = previousArea;
	}
}