package core;


/**
 * Imports
 */
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedList;


/**
 * World
 * @author Varkalas D`Lonovan
 */
public class World {
	/**
	 * Class Variables
	 */
	private LinkedList spells;
	private LinkedList items;
	private LinkedList npcs;
	private LinkedList world;
	private GameEntity player;


	/**
	 * Constructor
	 */
	public World() {
		spells = new LinkedList();
		items = new LinkedList();
		world = new LinkedList();
		npcs = new LinkedList();
		System.out.print("Loading World... ");
		loadWorld();
    	System.out.println("DONE");
    	System.out.print("Loading Spells... ");
    	loadSpells();
    	System.out.println("DONE");
    	System.out.print("Loading Items...");
    	loadItems();
    	System.out.println("DONE");
		System.out.print("Loading NPCs... ");
		loadNPCs();
    	System.out.println("DONE");
	}


	/**
	 * loadWorld
	 */
	public void loadWorld() {
		try {
			BufferedReader bufferedReader = new BufferedReader(new FileReader("Data/World.db"));
			String line = "";
			String data = "";

			// read from the file
			while ((line = bufferedReader.readLine()) != null) {
				Area area = new Area();
				int index = 0;

				// name
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				area.setName(data);
				data = "";
				index++;

				// load the mask for the area
				try {
					BufferedReader bufferedReader2 = new BufferedReader(new FileReader("Data/World/" + area.getName() + ".db"));
					String line2 = "";
					int index1 = 0;
					while ((line2 = bufferedReader2.readLine()) != null) {
						for (int index2 = 0; index2 < 20; index2++) {
							String temp = "" + line2.charAt(index2 * 2);
							area.setMask(Integer.parseInt(temp), index2, index1);
						}
						if (index1 < 15) {
							index1++;
						}
					}
				} catch (FileNotFoundException exception) {
		    		System.err.println("[World.java]  Could not find the file " + area.getName() + ".db.");
		    	} catch (IOException exception) {
		    		System.err.println("[World.java]  Unable to read from " + area.getName() + ".db.");
		    	}

				// above area
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				area.setAbove(data);
				data = "";
				index++;

				// below area
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				area.setBelow(data);
				data = "";
				index++;

				// left area
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				area.setLeft(data);
				data = "";
				index++;

				// right area
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				area.setRight(data);
				data = "";
				index++;

				// collect all spawn point information, if there is any
				while (line.charAt(index) != '~') {
					int x, y;

					// x loc
					while (line.charAt(index) != ',') {
						data += line.charAt(index);
						index++;
					}
					x = Integer.parseInt(data);
					data = "";
					index++;

					// y loc
					while (line.charAt(index) != '-') {
						data += line.charAt(index);
						index++;
					}
					y = Integer.parseInt(data);
					data = "";
					index++;

					// area name
					while (line.charAt(index) != ';') {
						data += line.charAt(index);
						index++;
					}
					area.setSpawnPoint(data, x, y);
					data = "";
					index++;
				}
				data = "";
				index++;

				// collect all entrance point information, if there is any
				while (line.charAt(index) != '~') {
					int x, y;

					// x loc
					while (line.charAt(index) != ',') {
						data += line.charAt(index);
						index++;
					}
					x = Integer.parseInt(data);
					data = "";
					index++;

					// y loc
					while (line.charAt(index) != '-') {
						data += line.charAt(index);
						index++;
					}
					y = Integer.parseInt(data);
					data = "";
					index++;

					// area name
					while (line.charAt(index) != ';') {
						data += line.charAt(index);
						index++;
					}
					area.setEntrancePoint(data, x, y);
					data = "";
					index++;
				}

				// add the area to the world
				world.add(area);
			}
    	} catch (FileNotFoundException exception) {
    		System.err.println("[World.java]  Could not find the file World.db.");
    	} catch (IOException exception) {
    		System.err.println("[World.java]  Unable to read from World.db.");
    	}
	}


	/**
	 * loadSpells
	 */
	public void loadSpells() {
		try {
			BufferedReader bufferedReader = new BufferedReader(new FileReader("Data/Spells.db"));
			String line = "";
			String data = "";

			// read from the file
			while ((line = bufferedReader.readLine()) != null) {
				int index = 0;
				Spell spell = new Spell();

				// name
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				spell.setName(data);
				data = "";
				index++;

				// type
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				spell.setType(data);
				data = "";
				index++;

				// intensity
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				spell.setIntensity(Integer.parseInt(data));
				data = "";
				index++;

				// add the spell to the list of spells
				spells.add(spell);
			}
		} catch (FileNotFoundException exception) {
    		System.err.println("[World.java]  Could not find the file Spells.db.");
    	} catch (IOException exception) {
    		System.err.println("[World.java]  Unable to read from Spells.db.");
    	}
	}


	/**
	 * loadItems
	 */
	public void loadItems() {
		try {
			BufferedReader bufferedReader = new BufferedReader(new FileReader("Data/Items.db"));
			String line = "";
			String data = "";

			while ((line = bufferedReader.readLine()) != null) {
				int index = 0;
				Item item = new Item();

				// name
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				item.setName(data);
				data = "";
				index++;

				// location
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				item.setBodyLocation(data);
				data = "";
				index++;

				// damage rating
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				item.setDamageRating(Integer.parseInt(data));
				data = "";
				index++;

				// attack speed
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				item.setAttackSpeed(Integer.parseInt(data));
				data = "";
				index++;

				// armor rating
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				item.setArmorRating(Integer.parseInt(data));
				data = "";
				index++;

				// clickable spell
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				item.setClickableSpell(getSpell(data));
				data = "";
				index++;

				// weapon effect
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				item.setWeaponEffect(getSpell(data));
				data = "";
				index++;

				// add the item to the list of items
				items.add(item);
			}
		} catch (FileNotFoundException exception) {
    		System.err.println("[World.java]  Could not find the file Items.db.");
    	} catch (IOException exception) {
    		System.err.println("[World.java]  Unable to read from Items.db.");
    	}
	}


	/**
	 * loadNPCs
	 */
	public void loadNPCs() {
		// go through all the areas
		for (int index1 = 0; index1 < world.size(); index1++) {
			// go through all the spawn points
			for (int index2 = 0; index2 < 20; index2++) {
				for (int index3 = 0; index3 < 15; index3++) {
					String spawn = ((Area) world.get(index1)).getSpawnPoints()[index2][index3];
					if (!spawn.equals("")) {
						// TODO: (this is flawed... need a unique ID) load the NPC
						GameEntity npc = loadNPC(spawn);

						// setup area and location
						npc.setCurrentArea((Area) world.get(index1));
						npc.setCurrentLocation(index2 * 32, index3 * 32);
						npc.setPreviousArea(npc.getCurrentArea());
						npc.setPreviousLocation(index2 * 32, index3 * 32);

						// add the npc to the npc list
						npcs.add(npc);
					}
				}
			}
		}

		// TODO: go through areas, look at spawn points, set them up
		
	}


	/**
	 * loadNPC
	 * @param name The name of the NPC to load.
	 */
	public GameEntity loadNPC(String name) {
		GameEntity newNPC = new GameEntity(this);

		try {
			BufferedReader bufferedReader = new BufferedReader(new FileReader("Data/NPCs.db"));
			String line = "";
			String data = "";

			// read from the file
			while ((line = bufferedReader.readLine()) != null) {
				int index = 0;

				// name
				while (line.charAt(index) != '~') {
					data += line.charAt(index);
					index++;
				}
				if (name.equals(data)) {
					// set some stuff
					newNPC.setName(data);
					newNPC.setSprite(data);
					data = "";
					index++;

					// profession
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.getProfession().setName(data);
					data = "";
					index++;

					// might
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.getAttributes().setMight(Integer.parseInt(data));
					data = "";
					index++;

					// endurance
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.getAttributes().setEndurance(Integer.parseInt(data));
					data = "";
					index++;

					// quickness
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.getAttributes().setQuickness(Integer.parseInt(data));
					data = "";
					index++;

					// acumen
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.getAttributes().setAcumen(Integer.parseInt(data));
					data = "";
					index++;

					// type
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.setType(data);
					data = "";
					index++;

					// movement rate
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.getAttributes().setMovementRate(Integer.parseInt(data));
					data = "";
					index++;

					// spells
					while (line.charAt(index) != '~') {
						// add each spell
						while (line.charAt(index) != ';') {
							data += line.charAt(index);
							index++;
						}
						newNPC.getProfession().getSpells().add(getSpell(data));
						data = "";
						index++;
					}
					data = "";
					index++;

					// items
					while (line.charAt(index) != '~') {
						String itemName;

						// get the item name
						while (line.charAt(index) != '(') {
							data += line.charAt(index);
							index++;
						}
						itemName = data;
						data = "";
						index++;

						// get the item chance
						while (line.charAt(index) != '%') {
							data += line.charAt(index);
							index++;
						}
						if (RandomChance.didChanceSucceed(Integer.parseInt(data))) {
							// add the item if the chance succeeds
							newNPC.getEquipment().addToInventory(getItem(itemName));
						}
						data = "";
						index++;

						// rotate back
						while (line.charAt(index) != ')') {
							index++;
						}
						while (line.charAt(index) != ';') {
							index++;
						}
						index++;
					}
					data = "";
					index++;

					// gold
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					newNPC.getAttributes().setAcumen(Integer.parseInt(data));
					data = "";
					index++;
				} else {
					data = "";
				}
			}
		} catch (FileNotFoundException exception) {
    		System.err.println("[World.java]  Could not find the file NPCs.db.");
    	} catch (IOException exception) {
    		System.err.println("[World.java]  Unable to read from NPCs.db.");
    	}

    	return newNPC;
	}


	/**
	 * getArea
	 * @param name The name of the area.
	 * @return Returns the area if it's in the current world.
	 */
	public Area getArea(String name) {
		// search the world for the area
		for (int index = 0; index < world.size(); index++) {
			if ((((Area) world.get(index)).getName()).equals(name)) {
				return ((Area) world.get(index));
			}
		}

		// didn't find the area
		return null;
	}


	/**
	 * getSpell
	 * @param name The name of the spell.
	 * @return Returns the spell.
	 */
	public Spell getSpell(String name) {
		// search the spell list for the spell
		for (int index = 0; index < spells.size(); index++) {
			if ((((Spell) spells.get(index)).getName()).equals(name)) {
				return ((Spell) spells.get(index));
			}
		}

		// didn't find the spell
		return null;
	}


	/**
	 * getItem
	 * @param name The name of the item.
	 * @return Returns the item.
	 */
	public Item getItem(String name) {
		// search the item list for the item
		for (int index = 0; index < items.size(); index++) {
			if ((((Item) items.get(index)).getName()).equals(name)) {
				return ((Item) items.get(index));
			}
		}

		// didn't find the item
		return null;
	}


	/**
	 * getNPC
	 * @param name The name of the NPC.
	 * @return Returns the NPC.
	 */
	public GameEntity getNPC(String name) {
		// TODO: this is a problem, as there are more than one creature for any given area
		// search the creature list for the creature
		for (int index = 0; index < npcs.size(); index++) {
			if ((((GameEntity) npcs.get(index)).getName()).equals(name)) {
				return ((GameEntity) npcs.get(index));
			}
		}

		// didn't find the area
		return null;
	}


	/**
	 * @return Returns a list of all the NPCs.
	 */
	public LinkedList getNPCs() {
		return npcs;
	}


	/**
	 * @return Returns the player.
	 */
	public GameEntity getPlayer() {
		return player;
	}


	/**
	 * @param player The player to set.
	 */
	public void setPlayer(GameEntity player) {
		this.player = player;
	}
}