package graphics;


/**
 * Imports
 */
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.ImageIcon;


/**
 * TitleGUI
 */
public class EngineTitleGUI extends GUI {
	/**
	 * Class Variables
	 */
	private Renderer renderer;
	private Image background;
	private Image torch;
	private int currentSelection;
	private String[] options = {"Map Maker",
			                    "Item Maker",
			                    "NPC Maker",
			                    "Spell Maker",
			                    "Quit Game"};


	/**
	 * Constructor
	 */
    public EngineTitleGUI(Renderer renderer) {
    	this.renderer = renderer;
    	background = (new ImageIcon("Images/Backgrounds/CharacterCreation.gif")).getImage();
    	torch = (new ImageIcon("Images/Screens/AnimatedTorch.gif")).getImage();
    	currentSelection = -1;
    }


    /**
     * mouseMoved
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseMoved(MouseEvent mouseEvent) {
        if (mouseEvent.getX() >= 135 && mouseEvent.getX() < 275) {
            if (mouseEvent.getY() >= 130 && mouseEvent.getY() < 150) {
            	currentSelection = 0;
            } else if (mouseEvent.getY() >= 200 && mouseEvent.getY() < 220) {
            	currentSelection = 1;
            } else if (mouseEvent.getY() >= 270 && mouseEvent.getY() < 290) {
            	currentSelection = 2;
            } else if (mouseEvent.getY() >= 340 && mouseEvent.getY() < 360) {
            	currentSelection = 3;
            } else if (mouseEvent.getY() >= 410 && mouseEvent.getY() < 430) {
            	currentSelection = 4;
            } else {
            	currentSelection = -1;
            }
        }
        if (mouseEvent.getX() >= 520 && mouseEvent.getX() < 550) {
        	if (mouseEvent.getY() >= 220 && mouseEvent.getY() < 270) {
        		currentSelection = 3;
        	}
        }
    }


    /**
     * mouseReleased
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseReleased(MouseEvent mouseEvent) {
    	if (currentSelection == 0) {
    		renderer.switchWindow(Renderer.MAP_MAKER_WINDOW);
    	} else if (currentSelection == 1) {
    		renderer.switchWindow(Renderer.ITEM_MAKER_WINDOW);
    	} else if (currentSelection == 4) {
    		renderer.setGameRunning(false);
    	}
    }


    /**
     * keyPressed
     * @param keyEvent The key that gets pressed.
     */
    public void keyPressed(KeyEvent keyEvent) {
    	if (keyEvent.getKeyCode() == KeyEvent.VK_UP  || keyEvent.getKeyCode() == KeyEvent.VK_KP_UP) {
    		if (currentSelection == 0) {
    			currentSelection = 2;
    		} else if (currentSelection == 1) {
    			currentSelection = 0;
    		} else if (currentSelection == 2) {
    			currentSelection = 1;
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_DOWN || keyEvent.getKeyCode() == KeyEvent.VK_KP_DOWN) {
    		if (currentSelection == 0) {
    			currentSelection = 1;
    		} else if (currentSelection == 1) {
    			currentSelection = 2;
    		} else if (currentSelection == 2) {
    			currentSelection = 0;
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER) {
    		if (currentSelection == 0) {
    			renderer.switchWindow(Renderer.CHARACTER_CREATION_WINDOW);
    		} else if (currentSelection == 1) {
    			renderer.switchWindow(Renderer.CONTINUE_CHARACTER_WINDOW);
    		} else if (currentSelection == 2) {
    			renderer.setGameRunning(false);
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_ESCAPE) {
    		renderer.setGameRunning(false);
    	}
    }


    /**
     * render
     * @param graphics The graphics display.
     */
    public void render(Graphics2D graphics) {
    	// member variables
    	int increment = 70;

    	// draw the background
    	graphics.drawImage(background, 0, 0, null);

    	// draw the title
    	graphics.setFont(graphics.getFont().deriveFont(60.0f));
    	graphics.setColor(Color.DARK_GRAY);
    	graphics.drawString("ALACRITY GAME ENGINE", 60, 70);

    	// draw the basic title options
    	graphics.setFont(graphics.getFont().deriveFont(32.0f));
    	for (int index = 0; index < options.length; index++) {
    		if (currentSelection == index) {
    			graphics.drawImage(torch, 70, 75 + (increment * index), null);
    			graphics.setColor(Color.RED);
    		} else {
    			graphics.setColor(Color.BLACK);
    		}
    		graphics.drawString(options[index], 135, 150 + (increment * index));
    	}
    }
}