package graphics;


/**
 * Imports
 */
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;


/**
 * TitleGUI
 */
public class MapMakerGUI extends GUI {
	/**
	 * Class Variables
	 */
	private Renderer renderer;
	private Image background;
	private String options[] = {"Load Sprite",
					            "Insert Spawn Point",
					            "Remove Spawn Point",
					            "Insert Entrance Point",
					            "Remove Entrance Point",
					            "Fill Map with Sprite",
					            "Clear Area",
					            "Clear Loaded Sprites",
					            "Save Area",
					            "Divide Image",
					            "Back to Engine",
					            "Load Area",
					            "Set Above Area",
					            "Set Below Area",
					            "Set Left Area",
					            "Set Right Area",};
	private int currentSelection;
	private BufferedImage images[];
	private int imageType[];
	private int imageCounter;
	private int selectedImage;
	private Image map[][];
	private int mask[][];
	private int currentXLocation;
	private int currentYLocation;
	private String mapName;
	private Image spawnPoint;
	boolean insertingSpawnPoint;
	boolean removingSpawnPoint;
	private String spawnPoints[][];
	private Image entrancePoint;
	boolean insertingEntrancePoint;
	boolean removingEntrancePoint;
	private String entrancePoints[][];
	private String relativeAreas[];
	private static final int ABOVE = 0;
	private static final int BELOW = 1;
	private static final int LEFT = 2;
	private static final int RIGHT = 3;


	/**
	 * Constructor
	 */
    public MapMakerGUI(Renderer renderer) {
    	this.renderer = renderer;
    	// NOTE:  Screen sizes that are divisible by 32x32 keeping the 4:3 ratio
    	//        128x96, 256x192, 384x288, 512x384, 640x480, 768x576, 896x672, 1024x768
    	renderer.setSize(896, 672);
        int screenWidth = Toolkit.getDefaultToolkit().getScreenSize().width;
        int screenHeight = Toolkit.getDefaultToolkit().getScreenSize().height;
        renderer.setLocation((screenWidth - 896) / 2, (screenHeight - 672) / 2);
    	background = (new ImageIcon("Images/GameEngine/MapMaker.gif")).getImage();
    	currentSelection = -1;
    	images = new BufferedImage[20];
    	imageType = new int[20];
    	imageCounter = 0;
    	selectedImage = -1;
    	map = new Image[20][15];
    	mask = new int[20][15];
    	currentXLocation = 0;
    	currentYLocation = 0;
    	mapName = "";
    	spawnPoint = (new ImageIcon("Images/GameEngine/SpawnPoint.gif")).getImage();
    	entrancePoint = (new ImageIcon("Images/GameEngine/EntrancePoint.gif")).getImage();
    	insertingSpawnPoint = false;
    	removingSpawnPoint = false;
    	insertingEntrancePoint = false;
    	removingEntrancePoint = false;
    	spawnPoints = new String[20][15];
    	entrancePoints = new String[20][15];
    	for (int index1 = 0; index1 < 20; index1++) {
    		for (int index2 = 0; index2 < 15; index2++) {
    			spawnPoints[index1][index2] = "";
    			entrancePoints[index1][index2] = "";
    		}
    	}
    	relativeAreas = new String[4];
    	for (int index = 0; index < relativeAreas.length; index++) {
    		relativeAreas[index] = "";
    	}
    }


    /**
     * mouseMoved
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseMoved(MouseEvent mouseEvent) {
    	// user options
    	if (mouseEvent.getX() >= 32 && mouseEvent.getX() < 190) {
    		if (!insertingSpawnPoint && !removingSpawnPoint && !insertingEntrancePoint && !removingEntrancePoint) {
	            if (mouseEvent.getY() >= 35 && mouseEvent.getY() < 50) {
	            	currentSelection = 0;
	            } else if (mouseEvent.getY() >= 85 && mouseEvent.getY() < 100) {
	            	currentSelection = 1;
	            } else if (mouseEvent.getY() >= 135 && mouseEvent.getY() < 150) {
	            	currentSelection = 2;
	            } else if (mouseEvent.getY() >= 185 && mouseEvent.getY() < 200) {
	            	currentSelection = 3;
	            } else if (mouseEvent.getY() >= 235 && mouseEvent.getY() < 250) {
	            	currentSelection = 4;
	            } else if (mouseEvent.getY() >= 285 && mouseEvent.getY() < 300) {
	            	currentSelection = 5;
	            } else if (mouseEvent.getY() >= 325 && mouseEvent.getY() < 350) {
	            	currentSelection = 6;
	            } else if (mouseEvent.getY() >= 385 && mouseEvent.getY() < 400) {
	            	currentSelection = 7;
	            } else if (mouseEvent.getY() >= 425 && mouseEvent.getY() < 450) {
	            	currentSelection = 8;
	            } else if (mouseEvent.getY() >= 485 && mouseEvent.getY() < 500) {
	            	currentSelection = 9;
	            } else if (mouseEvent.getY() >= 525 && mouseEvent.getY() < 550) {
	            	currentSelection = 10;
	            }
	            if (mouseEvent.getX() >= 32 && mouseEvent.getX() < 222) {
	                if (mouseEvent.getY() >= 585 && mouseEvent.getY() < 635) {
	                	currentSelection = 16;
	                }
	            }
    		}
    	} else if (mouseEvent.getY() >= 620 && mouseEvent.getY() < 635) {
    		if (mouseEvent.getX() >= 240 && mouseEvent.getX() < 310) {
    			currentSelection = 11;
    		} else if (mouseEvent.getX() >= 340 && mouseEvent.getX() < 455) {
    			currentSelection = 12;
    		} else if (mouseEvent.getX() >= 480 && mouseEvent.getX() < 590) {
    			currentSelection = 13;
    		} else if (mouseEvent.getX() >= 620 && mouseEvent.getX() < 720) {
    			currentSelection = 14;
    		} else if (mouseEvent.getX() >= 760 && mouseEvent.getX() < 860) {
    			currentSelection = 15;
    		} else {
    			currentSelection = -1;
    		}
    	} else if (mouseEvent.getX() >= 32 && mouseEvent.getX() < 222) {
            if (mouseEvent.getY() >= 585 && mouseEvent.getY() < 635) {
            	currentSelection = 16;
            }
    	} else {
    		currentSelection = -1;
    	}
    }


    /**
     * mouseDragged
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseDragged(MouseEvent mouseEvent) {
    	// map section
    	if (mouseEvent.getX() >= 224 && mouseEvent.getX() < 864) {
    		if (mouseEvent.getY() >= 32 && mouseEvent.getY() < 512) {
	    		for (int index1 = 0; index1 < 32; index1++) {
	        		for (int index2 = 0; index2 < 32; index2++) {
        				if (mouseEvent.getX() >= 224 + (32 * index1) && mouseEvent.getX() < 256 + (32 * index1)) {
        					if (mouseEvent.getY() >= 32 + (32 * index2) && mouseEvent.getY() < 64 + (32 * index2)) {
        						// sprites
        						if (selectedImage != -1) {
        							map[index1][index2] = images[selectedImage];
        							mask[index1][index2] = imageType[selectedImage];
        							currentXLocation = index1;
        							currentYLocation = index2;
        							incrementLocation();
        						}
        					}
        				}
	        		}
	        	}
    		}
    	}
    }


    /**
     * mouseReleased
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseReleased(MouseEvent mouseEvent) {
    	currentSelection = -1;

    	// user options
    	if (!insertingSpawnPoint && !removingSpawnPoint && !insertingEntrancePoint && !removingEntrancePoint) {
    		if (mouseEvent.getX() >= 32 && mouseEvent.getX() < 190) {
	            if (mouseEvent.getY() >= 35 && mouseEvent.getY() < 50) {
	            	loadSprite();
	            } else if (mouseEvent.getY() >= 85 && mouseEvent.getY() < 100) {
	            	insertingSpawnPoint = true;
	            } else if (mouseEvent.getY() >= 135 && mouseEvent.getY() < 150) {
	            	removingSpawnPoint = true;
	            } else if (mouseEvent.getY() >= 185 && mouseEvent.getY() < 200) {
	            	insertingEntrancePoint = true;
	            } else if (mouseEvent.getY() >= 235 && mouseEvent.getY() < 250) {
	            	removingEntrancePoint = true;
	            } else if (mouseEvent.getY() >= 285 && mouseEvent.getY() < 300) {
	            	fillMapWithCurrentSprite();
	            } else if (mouseEvent.getY() >= 325 && mouseEvent.getY() < 350) {
	            	clearMap();
	            } else if (mouseEvent.getY() >= 385 && mouseEvent.getY() < 400) {
	            	clearLoadedSprites();
	            } else if (mouseEvent.getY() >= 425 && mouseEvent.getY() < 450) {
	            	saveMap();
	            } else if (mouseEvent.getY() >= 485 && mouseEvent.getY() < 500) {
	            	divideImageIntoSprites();
	            } else if (mouseEvent.getY() >= 525 && mouseEvent.getY() < 550) {
	            	renderer.switchWindow(Renderer.ENGINE_TITLE_WINDOW);
	            }
    		} else if (mouseEvent.getY() >= 620 && mouseEvent.getY() < 635) {
	    		if (mouseEvent.getX() >= 240 && mouseEvent.getX() < 310) {
	    			loadArea();
	    		} else if (mouseEvent.getX() >= 340 && mouseEvent.getX() < 455) {
	    			setArea(ABOVE);
	    		} else if (mouseEvent.getX() >= 480 && mouseEvent.getX() < 590) {
	    			setArea(BELOW);
	    		} else if (mouseEvent.getX() >= 620 && mouseEvent.getX() < 720) {
	    			setArea(LEFT);
	    		} else if (mouseEvent.getX() >= 760 && mouseEvent.getX() < 860) {
	    			setArea(RIGHT);
	    		}
    		}
    	}

    	// map section
    	if (mouseEvent.getX() >= 224 && mouseEvent.getX() < 864) {
    		if (mouseEvent.getY() >= 32 && mouseEvent.getY() < 512) {
	    		for (int index1 = 0; index1 < 20; index1++) {
	        		for (int index2 = 0; index2 < 15; index2++) {
        				if (mouseEvent.getX() >= 224 + (32 * index1) && mouseEvent.getX() < 256 + (32 * index1)) {
        					if (mouseEvent.getY() >= 32 + (32 * index2) && mouseEvent.getY() < 64 + (32 * index2)) {
        						// spawn points and entrance points
        						if (insertingSpawnPoint) {
        							insertSpawnPoint(index1, index2);
        						} else if (removingSpawnPoint) {
        							removeSpawnPoint(index1, index2);
        						} else if (insertingEntrancePoint) {
        							insertEntrancePoint(index1, index2);
        						} else if (removingEntrancePoint) {
        							removeEntrancePoint(index1, index2);

        						// sprites
        						} else if (selectedImage != -1) {
        							map[index1][index2] = images[selectedImage];
        							mask[index1][index2] = imageType[selectedImage];
        							currentXLocation = index1;
        							currentYLocation = index2;
        							incrementLocation();
        						}
        					}
        				}
	        		}
	        	}
    		}
    	}

    	// clicked on a loaded sprite
    	for (int index = 0; index < images.length; index++) {
    		if (mouseEvent.getX() >= (240 + (index * 64)) && mouseEvent.getX() < (272 + (index * 64))) {
    			if (index < 10) {
	    			if (mouseEvent.getY() >= 528 && mouseEvent.getY() < 560) {
	    				selectedImage = index;
	    			}
    			} else {
    				if (mouseEvent.getY() >= 576 && mouseEvent.getY() < 608) {
	    				selectedImage = index;
	    			}
    			}
    		}
    	}
    }


    /**
     * keyPressed
     * @param keyEvent The key that gets pressed.
     */
    public void keyPressed(KeyEvent keyEvent) {
    	// typing in the name
    	if (currentSelection == 16) {
			if (keyEvent.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
				if (mapName.length() > 0) {
					mapName = mapName.substring(0, mapName.length() - 1);
				}
			} else if (mapName.length() <= 20 && keyEvent.getKeyCode() != KeyEvent.VK_BACK_SPACE
					                          && keyEvent.getKeyCode() != KeyEvent.VK_TAB
					                          && keyEvent.getKeyCode() != KeyEvent.VK_SPACE
					                          && keyEvent.getKeyCode() != KeyEvent.VK_SHIFT) {
				mapName += keyEvent.getKeyChar();
			}
		}

    	// inserting a sprite via spacebar
    	if (keyEvent.getKeyCode() == KeyEvent.VK_SPACE) {
    		if (selectedImage != -1) {
    			map[currentXLocation][currentYLocation] = images[selectedImage];
    			mask[currentXLocation][currentYLocation] = imageType[selectedImage];
    			incrementLocation();
    		}
    	}

    	// exiting the mapmaker
    	if (keyEvent.getKeyCode() == KeyEvent.VK_ESCAPE) {
    		renderer.switchWindow(Renderer.ENGINE_TITLE_WINDOW);
    	}
    }


    /**
     * loadSprite
     */
    public void loadSprite() {
    	JFileChooser fileChooser = new JFileChooser("Images/Sprites/");
		int selection = fileChooser.showOpenDialog(renderer);
	    if (selection == JFileChooser.APPROVE_OPTION) {
	    	String pathName = fileChooser.getCurrentDirectory().getPath(); 
	    	String imageName = fileChooser.getSelectedFile().getName();
	    	images[imageCounter] = new BufferedImage(32, 32, BufferedImage.TYPE_INT_ARGB);
	    	try {
	    		images[imageCounter] = ImageIO.read(new File(pathName + "\\" + imageName));
	    	} catch (IOException exception) {
	    		System.err.println("[MapMakerGUI.java]  Could not read " + imageName + ".");
	    	}
	    	if (imageName.contains("Mountain") || imageName.contains("Volcano")
	    	 || imageName.contains("Rocks") || imageName.contains("Wall")) {
	    		imageType[imageCounter] = 1;
	    	} else if (imageName.contains("Water")) {
	    		imageType[imageCounter] = 2;
	    	} else {
	    		imageType[imageCounter] = 0;
	    	}
	    	selectedImage = imageCounter;
	    } else {
	    	return;
	    }

	    if (imageCounter == 19) {
	    	JOptionPane.showConfirmDialog(renderer,
					                      "There is a maximum of 20 sprites loaded at a time.",
					                      "Error",
					                      JOptionPane.DEFAULT_OPTION,
					                      JOptionPane.ERROR_MESSAGE);
	    } else {
	    	imageCounter++;
	    }
    }


    /**
     * fillMapWithCurrentSprite
     */
    public void fillMapWithCurrentSprite() {
	    if (selectedImage != -1) {
	    	for (int index1 = 0; index1 < 20; index1++) {
	    		for (int index2 = 0; index2 < 15; index2++) {
	    			map[index1][index2] = images[selectedImage];
	    			mask[index1][index2] = imageType[selectedImage];
	    			incrementLocation();
	    		}
	    	}
	    	currentXLocation = 0;
			currentYLocation = 0;
	    } else {
	    	JOptionPane.showConfirmDialog(renderer,
					                      "No sprite has been selected.",
					                      "Error",
					                      JOptionPane.DEFAULT_OPTION,
					                      JOptionPane.ERROR_MESSAGE);
	    }
    }


    /**
     * clearLoadedSprites
     */
    public void clearLoadedSprites() {
    	for (int index = 0; index < images.length; index++) {
    		images[index] = null;
    		imageType[index] = 0;
    	}
    	imageCounter = 0;
    	selectedImage = -1;
    }


    /**
     * clearMap
     */
    public void clearMap() {
    	for (int index1 = 0; index1 < 20; index1++) {
    		for (int index2 = 0; index2 < 15; index2++) {
    			map[index1][index2] = null;
    			mask[index1][index2] = 0;
    			spawnPoints[index1][index2] = "";
    			entrancePoints[index1][index2] = "";
    		}
    	}
    	currentXLocation = 0;
    	currentYLocation = 0;
    }


    /**
     * saveMap
     */
    public void saveMap() {
    	// member variables
    	BufferedImage wholeMap = new BufferedImage(640, 480, BufferedImage.TYPE_INT_RGB);
    	LinkedList world = new LinkedList();
		BufferedReader bufferedReader;
		BufferedWriter bufferedWriter;
		String line = "";

    	// go through the map
    	for (int index1 = 0; index1 < 20; index1++) {
    		for (int index2 = 0; index2 < 15; index2++) {
    			// check for a name
    			if (mapName.equals("")) {
    				JOptionPane.showConfirmDialog(renderer,
							                      "The map requires a name.",
							                      "Error",
							                      JOptionPane.DEFAULT_OPTION,
							                      JOptionPane.ERROR_MESSAGE);
    				return;
    			}

    			// create the map image
    			wholeMap.createGraphics().drawImage(map[index1][index2], index1 * 32, index2 * 32, null); 
    		}
    	}

    	// save map images
    	try {
    		ImageIO.write(wholeMap, "jpg", new File("Images/World/" + mapName + ".jpg"));
    	} catch (IOException exception) {
    		System.err.println("[MapMakerGUI.java]  Could not write the map to " + mapName + ".jpg.");
    	}

		// load up old information
		try {
			bufferedReader = new BufferedReader(new FileReader("Data/World.db"));
			while ((line = bufferedReader.readLine()) != null) {
				int index = 0;
				while (line.charAt(index) != '~') {
					index++;
				}

				// add every other line but the one that has the name of this map
				if (!(line.substring(0, index)).equals(mapName)) {
					world.add(line);
				}
			}
		} catch (FileNotFoundException exception) {
    		System.err.println("[MapMakerGUI.java]  Could not find the file World.db.");
    	} catch (IOException exception) {
    		System.err.println("[MapMakerGUI.java]  Unable to read from World.db.");
    	}

		// setup the new information
		line = mapName + "~";
		line += relativeAreas[ABOVE] + "~";
		line += relativeAreas[BELOW] + "~";
		line += relativeAreas[LEFT] + "~";
		line += relativeAreas[RIGHT] + "~";
		for (int index1 = 0; index1 < 20; index1++) {
    		for (int index2 = 0; index2 < 15; index2++) {
    			if (!spawnPoints[index1][index2].equals("")) {
    				line += Integer.toString(index1) + "," + Integer.toString(index2) + "-" + spawnPoints[index1][index2] + ";";
    			}
    		}
		}
		line += "~";
		for (int index1 = 0; index1 < 20; index1++) {
    		for (int index2 = 0; index2 < 15; index2++) {
    			if (!entrancePoints[index1][index2].equals("")) {
    				line += Integer.toString(index1) + "," + Integer.toString(index2) + "-" + entrancePoints[index1][index2] + ";";
    			}
    		}
		}
		line += "~";
		world.add(line);

		// write the mask to a file
		try {
			bufferedWriter = new BufferedWriter(new FileWriter("Data/World/" + mapName + ".db"));
			for (int index1 = 0; index1 < 15; index1++) {
	    		for (int index2 = 0; index2 < 20; index2++) {
	    			bufferedWriter.write(Integer.toString(mask[index2][index1]));
	    			if (index2 < 19) {
	    				bufferedWriter.write(",");
	    			}
	    		}
	    		if (index1 < 14) {
	    			bufferedWriter.newLine();
	    		}
			}

			bufferedWriter.close();
		} catch (IOException exception) {
			System.err.println("[MapMakerGUI.java]  Could not write the " + mapName + ".db file.");
		}

		// write all information
		try {
			bufferedWriter = new BufferedWriter(new FileWriter("Data/World.db"));
			for (int index = 0; index < world.size(); index++) {
				bufferedWriter.write((String) world.get(index));
				bufferedWriter.newLine();
			}
			bufferedWriter.close();
		} catch (IOException exception) {
			System.err.println("[MapMakerGUI.java]  Could not read in the World.db file.");
		}
    }


    /**
     * divideImageIntoSprites
     */
    public void divideImageIntoSprites() {
    	BufferedImage bufferedImage = null;
    	String imageName = "";

    	// load up an image
    	JFileChooser fileChooser = new JFileChooser("Images/Angband Graphics/");
		int selection = fileChooser.showOpenDialog(renderer);
	    if (selection == JFileChooser.APPROVE_OPTION) {
	    	String currentDirectory = fileChooser.getCurrentDirectory().getPath();
	    	imageName =  fileChooser.getSelectedFile().getName();
	    	try {
	    		bufferedImage = ImageIO.read(new File(currentDirectory + "\\" + imageName));
	    	} catch(IOException exception) {
	    		System.err.println("[MapMakerGUI.java]  Could not open the file " + fileChooser.getSelectedFile().getName() + ".");
	    	}
	    } else {
	    	return;
	    }

	    // go through the image
	    for (int index1 = 0; index1 < bufferedImage.getHeight(); index1 = index1 + 32) {
	    	for (int index2 = 0; index2 < bufferedImage.getWidth(); index2 = index2 + 32) {
	    		// setup the colors for the 32x32 sprite
	    		BufferedImage sprite = new BufferedImage(32, 32, BufferedImage.TYPE_INT_RGB);
			    for (int index3 = 0; index3 < 32; index3++) {
			    	for (int index4 = 0; index4 < 32; index4++) {
			    		sprite.setRGB(index3, index4, bufferedImage.getRGB(index2 + index3, index1 + index4));
			    	}
			    }

			    // save the sprite
			    String newImageName = imageName.substring(0, imageName.length() - 4);
			    try {
		    		ImageIO.write(sprite, "bmp", new File("Images/Sprites/Not Categorized/" + newImageName + (index1 / 32) + (index2 / 32) + ".bmp"));
		    	} catch (IOException exception) {
		    		System.err.println("[MapMakerGUI.java]  Could not write the sprite to " + "Images/Sprites/New/" + newImageName + (index1 / 32) + (index2 / 32) + ".jpg.");
		    	}
	    	}
	    }
    }


    /**
     * loadArea
     */
    public void loadArea() {
    	String imageName = "";
    	BufferedImage bufferedMap = new BufferedImage(640, 480, BufferedImage.TYPE_INT_RGB);

    	// load up the image
    	JFileChooser fileChooser = new JFileChooser("Images/World/");
		int selection = fileChooser.showOpenDialog(renderer);
	    if (selection == JFileChooser.APPROVE_OPTION) {
	    	try {
	    		String maskName;
	    		imageName = fileChooser.getSelectedFile().getName();
	    		mapName = imageName.substring(0, imageName.length() - 4);
	    		maskName = imageName.substring(0, imageName.length() - 4);
	    		maskName += "Mask.jpg";
	    		bufferedMap = ImageIO.read(new File(fileChooser.getCurrentDirectory().getPath() + "\\" + imageName));
	    	} catch (IOException exception) {
	    		System.err.println("[MapMakerGUI.java]  Could not read in the area " + imageName + ".");
	    	}
	    } else {
	    	return;
	    }

	    // clear the map
    	clearMap();

	    // go through the map
	    for (int index1 = 0; index1 < 20; index1++) {
	    	for (int index2 = 0; index2 < 15; index2++) {
	    		// load the 32x32 sprite into the map
	    		BufferedImage smallerImage = new BufferedImage(32, 32, BufferedImage.TYPE_INT_RGB);
	    		for (int index3 = 0; index3 < 32; index3++) {
	    			for (int index4 = 0; index4 < 32; index4++) {
	    				smallerImage.setRGB(index3, index4, bufferedMap.getRGB(index3 + (index1 * 32), index4 + (index2 * 32)));
	    			}
	    		}
	    		map[index1][index2] = (Image) smallerImage;
	    	}
	    }

		// load the mask
		try {
			BufferedReader bufferedReader = new BufferedReader(new FileReader("Data/World/" + mapName + ".db"));
			String line = "";
			int index1 = 0;
			while ((line = bufferedReader.readLine()) != null) {
				for (int index2 = 0; index2 < 20; index2++) {
					String temp = "" + line.charAt(index2 * 2);
					mask[index2][index1] = Integer.parseInt(temp);
				}
				if (index1 < 15) {
					index1++;
				}
			}
		} catch (FileNotFoundException exception) {
			System.err.println("[MapMakerGUI.java]  " + mapName + ".db does not exist.");
		} catch (IOException exception) {
			System.err.println("[MapMakerGUI.java]  Could not read " + mapName + ".db.");
		}

		// load the area's file
		try {
    		BufferedReader bufferedReader = new BufferedReader(new FileReader("Data/World.db"));
    		String line = "";
			while ((line = bufferedReader.readLine()) != null) {
				int index = 0;
				while (line.charAt(index) != '~') {
					index++;
				}
				String name = line.substring(0, index);
				index++;
				if (name.equals(imageName.substring(0, imageName.length() - 4))) {
					String data = "";

					// above area
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					relativeAreas[ABOVE] = data;
					data = "";
					index++;

					// below area
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					relativeAreas[BELOW] = data;
					data = "";
					index++;

					// left area
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					relativeAreas[LEFT] = data;
					data = "";
					index++;

					// right area
					while (line.charAt(index) != '~') {
						data += line.charAt(index);
						index++;
					}
					relativeAreas[RIGHT] = data;
					data = "";
					index++;

					// spawn point information
					while (line.charAt(index) != '~') {
						int x, y;

						// x
						while (line.charAt(index) != ',') {
							data += line.charAt(index);
							index++;
						}
						x = Integer.parseInt(data);
						data = "";
						index++;

						// y
						while (line.charAt(index) != '-') {
							data += line.charAt(index);
							index++;
						}
						y = Integer.parseInt(data);
						data = "";
						index++;

						// name
						while (line.charAt(index) != ';') {
							data += line.charAt(index);
							index++;
						}
						spawnPoints[x][y] = data;
						data = "";
						index++;
					}
					data = "";
					index++;

					// entrance point information
					while (line.charAt(index) != '~') {
						int x, y;

						// x
						while (line.charAt(index) != ',') {
							data += line.charAt(index);
							index++;
						}
						x = Integer.parseInt(data);
						data = "";
						index++;

						// y
						while (line.charAt(index) != '-') {
							data += line.charAt(index);
							index++;
						}
						y = Integer.parseInt(data);
						data = "";
						index++;

						// name
						while (line.charAt(index) != ';') {
							data += line.charAt(index);
							index++;
						}
						entrancePoints[x][y] = data;
						data = "";
						index++;
					}
				}
			}
		} catch (FileNotFoundException exception) {
			System.err.println("[MapMakerGUI.java]  World.db does not exist.");
		} catch (IOException exception) {
			System.err.println("[MapMakerGUI.java]  Could not read World.db.");
		}
    }


    /**
     * setArea
     * @param direction The placement with respect to the current area.
     */
    public void setArea(int direction) {
    	LinkedList world = new LinkedList();
		BufferedReader bufferedReader;
		BufferedWriter bufferedWriter;
		String line = "";
		String newLine = "";
    	String relatedArea = "";

	    // get the area to be set with respect to the other area
	    JFileChooser fileChooser = new JFileChooser("Images/World/");
	    int selection = fileChooser.showOpenDialog(renderer);
	    if (selection == JFileChooser.APPROVE_OPTION) {
	    	String temp = fileChooser.getSelectedFile().getName();
	    	relatedArea = temp.substring(0, temp.length() - 4);
	    } else {
	    	return;
	    }

	    // add or change the set area to the respective area
		try {
			bufferedReader = new BufferedReader(new FileReader("Data/World.db"));
			while ((line = bufferedReader.readLine()) != null) {
				int index = 0;
				while (line.charAt(index) != '~') {
					index++;
				}
				if (!(line.substring(0, index)).equals(mapName)) {
					world.add(line);
				} else {
					index++;
					if (direction == ABOVE) {
						relativeAreas[ABOVE] = relatedArea;
					} else if (direction == BELOW) {
						relativeAreas[BELOW] = relatedArea;
						while (line.charAt(index) != '~') {
							index++;
						}
						index++;
					} else if (direction == LEFT) {
						relativeAreas[LEFT] = relatedArea;
						while (line.charAt(index) != '~') {
							index++;
						}
						index++;
						while (line.charAt(index) != '~') {
							index++;
						}
						index++;
					} else if (direction == RIGHT) {
						relativeAreas[RIGHT] = relatedArea;
						while (line.charAt(index) != '~') {
							index++;
						}
						index++;
						while (line.charAt(index) != '~') {
							index++;
						}
						index++;
						while (line.charAt(index) != '~') {
							index++;
						}
						index++;
					}
					newLine = line.substring(0, index);
					newLine += relatedArea;
					while (line.charAt(index) != '~') {
						index++;
					}
					newLine += line.substring(index, line.length());
					world.add(newLine);
				}
			}
			bufferedReader.close();
		} catch (FileNotFoundException exception) {
    		System.err.println("[MapMakerGUI.java]  Could not find the file World.db.");
    	} catch (IOException exception) {
    		System.err.println("[MapMakerGUI.java]  Unable to read from World.db.");
    	}

    	// write the information
		try {
			bufferedWriter = new BufferedWriter(new FileWriter("Data/World.db"));
			for (int index = 0; index < world.size(); index++) {
				bufferedWriter.write((String) world.get(index));
				bufferedWriter.newLine();
			}
			bufferedWriter.close();
		} catch (IOException exception) {
			System.err.println("[MapMakerGUI.java]  Could not read in the World.db file.");
		}
    }


    /**
     * incrementLocation
     */
    public void incrementLocation() {
    	if (currentXLocation < 19) {
    		currentXLocation++;
    	} else {
    		currentXLocation = 0;
    		if (currentYLocation < 14) {
    			currentYLocation++;
    		} else {
    			currentYLocation = 0;
    		}
    	}
    }


    /**
     * insertSpawnPoint
     * @param x The x coordinate of the spawn point.
     * @param y The y coordinate of the spawn point.
     */
    public void insertSpawnPoint(int x, int y) {
		JFileChooser fileChooser = new JFileChooser("Images/Sprites/NPCs/");
		int selection = fileChooser.showOpenDialog(renderer);
	    if (selection == JFileChooser.APPROVE_OPTION) {
	    	if (!spawnPoints[x][y].equals("")) {
	    		spawnPoints[x][y] += ",";
	    	}
	    	String name = fileChooser.getSelectedFile().getName();
	    	spawnPoints[x][y] += name.substring(0, name.length() - 4);
			insertingSpawnPoint = false;
	    }
    }


    /**
     * removeSpawnPoint
     * @param x The x coordinate of the spawn point.
     * @param y The y coordinate of the spawn point.
     */
    public void removeSpawnPoint(int x, int y) {
    	spawnPoints[x][y] = "";
    	removingSpawnPoint = false;
    }


    /**
     * insertEntrancePoint
     * @param x The x coordinate of the entrance point.
     * @param y The y coordinate of the entrance point.
     */
    public void insertEntrancePoint(int x, int y) {
    	JFileChooser fileChooser = new JFileChooser("Images/World/");
		int selection = fileChooser.showOpenDialog(renderer);
	    if (selection == JFileChooser.APPROVE_OPTION) {
	    	String name = fileChooser.getSelectedFile().getName();
	    	entrancePoints[x][y] += name.substring(0, name.length() - 4);
	    	mask[x][y] = 2;
	    	insertingEntrancePoint = false;
	    }
    }


    /**
     * removeEntrancePoint
     * @param x The x coordinate of the entrance point.
     * @param y The y coordinate of the entrance point.
     */
    public void removeEntrancePoint(int x, int y) {
    	entrancePoints[x][y] = "";
		removingEntrancePoint = false;
    }


    /**
     * render
     * @param graphics The graphics display.
     */
    public void render(Graphics2D graphics) {
    	// member variables
    	int verticalIncrement = 50;
    	int horizontalIncrement = 64;

    	// draw the background
    	graphics.setColor(Color.BLACK);
    	graphics.drawImage(background, 0, 0, null);

    	// draw the options
    	graphics.setFont(graphics.getFont().deriveFont(20.0f));
    	for (int index = 0; index < options.length; index++) {
    		if (currentSelection == index) {
        		graphics.setColor(Color.RED);
        	} else {
        		graphics.setColor(Color.GREEN);
        	}
    		if (index < 11) {
    			graphics.drawString(options[index], 32, 50 + (verticalIncrement * index));
    		} else {
    			if (index == 11) {
    				graphics.drawString(options[index], 240, 635);
    			} else {
    				graphics.drawString(options[index], 340 + ((index - 12) * 140), 635);
    			}
    		}
    	}

    	// map name
    	graphics.setColor(Color.WHITE);
    	graphics.drawString("Map Name:", 32, 596);
    	if (currentSelection == 16) {
    		graphics.setColor(Color.RED);
    	}
    	graphics.drawRect(32, 610, 180, 25);
    	graphics.setFont(graphics.getFont().deriveFont(20.0f));
    	graphics.setColor(Color.YELLOW);
    	graphics.drawString(mapName, 35, 630);
    	
    	// draw the loaded sprites
    	for (int index = 0; index < images.length; index++) {
    		if (images[index] != null) {
    			BufferedImage loadedSprite = new BufferedImage(32, 32, BufferedImage.TYPE_INT_ARGB);
    	    	loadedSprite.createGraphics().drawImage((Image) images[index], 0, 0, null);
    			if (index < 10) {
    				graphics.drawImage((Image) loadedSprite, 240 + (index * horizontalIncrement), 528, null);
    				if (selectedImage == index) {
	    				graphics.setColor(Color.RED);
	    				graphics.drawRect(239 + (index * horizontalIncrement), 527, 34, 34);
    				}
    			} else {
    				graphics.drawImage((Image) loadedSprite, -400 + (index * horizontalIncrement), 576, null);
    				if (selectedImage == index) {
						graphics.setColor(Color.RED);
						graphics.drawRect(-399 + (index * horizontalIncrement), 575, 34, 34);
    				}
    			}
    		}
    	}

    	// draw the map
    	for (int index1 = 0; index1 < 20; index1++) {
    		for (int index2 = 0; index2 < 15; index2++) {
    			if (map[index1][index2] != null) {
    				graphics.drawImage((Image) map[index1][index2], 224 + (32 * index1), 32 + (32 * index2), null);
    			} else {
    				graphics.setColor(Color.BLACK);
    				graphics.fillRect(224 + (32 * index1), 32 + (32 * index2), 32, 32);
    			}
    		}
    	}

    	// draw the spawn and entrance points
    	for (int index1 = 0; index1 < 20; index1++) {
    		for (int index2 = 0; index2 < 15; index2++) {
    			if (!entrancePoints[index1][index2].equals("")) {
    				graphics.drawImage(entrancePoint, 224 + (32 * index1) + 8, 32 + (32 * index2) + 8, null);
    			}
    			if (!spawnPoints[index1][index2].equals("")) {
    				graphics.drawImage(spawnPoint, 224 + (32 * index1) + 8, 32 + (32 * index2) + 8, null);
    			}
    		}
    	}

    	// draw the insert entrance point icon
    	if (insertingEntrancePoint) {
    		if (renderer.getMousePosition() != null) {
	    		if (renderer.getMousePosition().getX() >= 8 && renderer.getMousePosition().getY() >= 8) {
	    			graphics.drawImage(entrancePoint, (int) renderer.getMousePosition().getX() - 8, (int) renderer.getMousePosition().getY() - 8, null);
	    		}
    		}
    	}

    	// draw the insert spawn point icon
    	if (insertingSpawnPoint) {
    		if (renderer.getMousePosition() != null) {
	    		if (renderer.getMousePosition().getX() >= 8 && renderer.getMousePosition().getY() >= 8) {
	    			graphics.drawImage(spawnPoint, (int) renderer.getMousePosition().getX() - 8, (int) renderer.getMousePosition().getY() - 8, null);
	    		}
    		}
    	}

    	// draw the remove spawn point and remove entrance point icon
    	if (removingSpawnPoint || removingEntrancePoint) {
    		if (renderer.getMousePosition() != null) {
	    		if (renderer.getMousePosition().getX() >= 8 && renderer.getMousePosition().getY() >= 8) {
	    			graphics.setFont(graphics.getFont().deriveFont(20.0f));
	    	    	graphics.setColor(Color.RED);
	    			graphics.drawString("X", (int) renderer.getMousePosition().getX() - 5, (int) renderer.getMousePosition().getY() + 5);
	    		}
    		}
    	}
    }
}