package graphics;


/**
 * Imports
 */
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.ImageIcon;


/**
 * TitleGUI
 */
public class TitleGUI extends GUI {
	/**
	 * Class Variables
	 */
	private Renderer renderer;
	private Image background;
	private Image torch;
	private int currentSelection;
	private String[] options = {"Create a New Character",
			                    "Continue a Previous Character",
			                    "Quit Game"};


	/**
	 * Constructor
	 */
    public TitleGUI(Renderer renderer) {
    	this.renderer = renderer;
    	background = (new ImageIcon("Images/Backgrounds/Title.gif")).getImage();
    	torch = (new ImageIcon("Images/Backgrounds/AnimatedTorch.gif")).getImage();
    	currentSelection = -1;
    }


    /**
     * mouseMoved
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseMoved(MouseEvent mouseEvent) {
        if (mouseEvent.getX() >= 135 && mouseEvent.getX() < 470) {
            if (mouseEvent.getY() >= 155 && mouseEvent.getY() < 175) {
            	currentSelection = 0;
            } else if (mouseEvent.getY() >= 275 && mouseEvent.getY() < 295) {
            	currentSelection = 1;
            } else if (mouseEvent.getY() >= 395 && mouseEvent.getY() < 415) {
            	currentSelection = 2;
            } else {
            	currentSelection = -1;
            }
        }
    }


    /**
     * mouseReleased
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseReleased(MouseEvent mouseEvent) {
    	if (mouseEvent.getX() >= 135 && mouseEvent.getX() < 470) {
            if (mouseEvent.getY() >= 155 && mouseEvent.getY() < 175) {
            	renderer.switchWindow(Renderer.CHARACTER_CREATION_WINDOW);
            } else if (mouseEvent.getY() >= 275 && mouseEvent.getY() < 295) {
            	renderer.switchWindow(Renderer.CONTINUE_CHARACTER_WINDOW);
            } else if (mouseEvent.getY() >= 395 && mouseEvent.getY() < 415) {
            	renderer.setGameRunning(false);
            }
        }
        if (mouseEvent.getX() >= 520 && mouseEvent.getX() < 550) {
        	if (mouseEvent.getY() >= 220 && mouseEvent.getY() < 270) {
        		renderer.switchWindow(Renderer.MAP_MAKER_WINDOW);
        	}
        }
    }


    /**
     * keyPressed
     * @param keyEvent The key that gets pressed.
     */
    public void keyPressed(KeyEvent keyEvent) {
    	if (keyEvent.getKeyCode() == KeyEvent.VK_UP  || keyEvent.getKeyCode() == KeyEvent.VK_KP_UP) {
    		if (currentSelection == 0) {
    			currentSelection = 2;
    		} else if (currentSelection == 1) {
    			currentSelection = 0;
    		} else if (currentSelection == 2) {
    			currentSelection = 1;
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_DOWN || keyEvent.getKeyCode() == KeyEvent.VK_KP_DOWN) {
    		if (currentSelection == 0) {
    			currentSelection = 1;
    		} else if (currentSelection == 1) {
    			currentSelection = 2;
    		} else if (currentSelection == 2) {
    			currentSelection = 0;
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_ENTER) {
    		if (currentSelection == 0) {
    			renderer.switchWindow(Renderer.CHARACTER_CREATION_WINDOW);
    		} else if (currentSelection == 1) {
    			renderer.switchWindow(Renderer.CONTINUE_CHARACTER_WINDOW);
    		} else if (currentSelection == 2) {
    			renderer.setGameRunning(false);
    		}
    	}
    }


    /**
     * render
     * @param graphics The graphics display.
     */
    public void render(Graphics2D graphics) {
    	// member variables
    	int increment = 120;

    	// draw the background
    	graphics.drawImage(background, 0, 0, null);

    	// draw the title
    	graphics.setFont(graphics.getFont().deriveFont(76.0f));
    	graphics.setColor(Color.CYAN);
    	graphics.drawString("MYSTIC REALMS", 110, 90);

    	// draw the basic title options
    	graphics.setFont(graphics.getFont().deriveFont(32.0f));
    	for (int index = 0; index < options.length; index++) {
    		if (currentSelection == index) {
    			graphics.drawImage(torch, 70, 100 + (increment * index), null);
    			graphics.setColor(Color.RED);
    		} else {
    			graphics.setColor(Color.GREEN);
    		}
    		graphics.drawString(options[index], 135, 175 + (increment * index));
    	}
    }
}