package graphics;


/**
 * Imports
 */
import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.Random;

import javax.swing.ImageIcon;

import core.Area;
import core.GameEntity;


/**
 * TitleGUI
 */
public class WorldGUI extends GUI {
	/**
	 * Class Variables
	 */
	private Renderer renderer;
	private Image topLeft;
	private Image top;
	private Image topRight;
	private Image left;
	private Image center;
	private Image right;
	private Image bottomLeft;
	private Image bottom;
	private Image bottomRight;
	private boolean enteringArea;
	private float timeCounter;
	private float creatureTimeCounter;
	private float creatureMovementCounter;


	/**
	 * Constructor
	 */
    public WorldGUI(Renderer renderer) {
    	this.renderer = renderer;

    	enteringArea = false;
    	timeCounter = 0;
    	creatureTimeCounter = 0;
    	creatureMovementCounter = 0;

    	// area images
    	loadImages();
    }


    /**
     * mouseMoved
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseMoved(MouseEvent mouseEvent) {
    }


    /**
     * mouseReleased
     * @param mouseEvent The event the mouse goes through.
     */
    public void mouseReleased(MouseEvent mouseEvent) {
    	// NPC selection
    	if (mouseEvent.getX() >= 0 && mouseEvent.getX() < 640) {
    		if (mouseEvent.getY() >= 0 && mouseEvent.getY() < 480) {
	    		for (int index1 = 0; index1 < 20; index1++) {
	        		for (int index2 = 0; index2 < 15; index2++) {
        				if (mouseEvent.getX() >= 0 + (32 * index1) - 16 && mouseEvent.getX() < 32 + (32 * index1) - 16) {
        					if (mouseEvent.getY() >= 0 + (32 * index2) && mouseEvent.getY() < 32 + (32 * index2)) {
        						for (int index = 0; index < renderer.getGame().getWorld().getNPCs().size(); index++) {
        				    		GameEntity npc = (GameEntity) renderer.getGame().getWorld().getNPCs().get(index);
        				    		Area centerArea = renderer.getGame().getCharacter().getCurrentArea();
        				    		Area topArea = renderer.getGame().getWorld().getArea(centerArea.getAbove());
        				    		Area bottomArea = renderer.getGame().getWorld().getArea(centerArea.getBelow());
        				    		int x = -1;
        				    		int y = -1;
        				    		int playerX = renderer.getGame().getCharacter().getCurrentLocation().getX();
        				    		int playerY = renderer.getGame().getCharacter().getCurrentLocation().getY();
        				    		int npcX = npc.getCurrentLocation().getX();
        				    		int npcY = npc.getCurrentLocation().getY();
        				    		if (renderer.getGame().getWorld().getArea(topArea.getLeft()).equals(npc.getCurrentArea())) {
        				    			x = (npcX + playerX - 640) / 32;
	        				    		y = (npcY + playerY - 480) / 32;
         				    		} else if (topArea.equals(npc.getCurrentArea())) {
         				    			x = (npcX + playerX) / 32;
	        				    		y = (npcY + playerY - 480) / 32;
         				    		} else if (renderer.getGame().getWorld().getArea(topArea.getRight()).equals(npc.getCurrentArea())) {
         				    			x = (npcX + playerX + 640) / 32;
	        				    		y = (npcY + playerY - 480) / 32;
         				    		} else if (renderer.getGame().getWorld().getArea(centerArea.getLeft()).equals(npc.getCurrentArea())) {
         				    			x = (npcX + playerX - 640) / 32;
	        				    		y = (npcY + playerY) / 32;
         				    		} else if (centerArea.equals(npc.getCurrentArea())) {
	        				    		x = (npcX + playerX) / 32;
	        				    		y = (npcY + playerY) / 32;
        				    		} else if (renderer.getGame().getWorld().getArea(centerArea.getRight()).equals(npc.getCurrentArea())) {
        				    			x = (npcX + playerX + 640) / 32;
        				    			y = (npcY + playerY) / 32;
        				    		} else if (renderer.getGame().getWorld().getArea(bottomArea.getLeft()).equals(npc.getCurrentArea())) {
        				    			x = (npcX + playerX - 640) / 32;
	        				    		y = (npcY + playerY + 480) / 32;
        				    		} else if (bottomArea.equals(npc.getCurrentArea())) {
        				    			x = (npcX + playerX) / 32;
	        				    		y = (npcY + playerY + 480) / 32;
        				    		} else if (renderer.getGame().getWorld().getArea(bottomArea.getRight()).equals(npc.getCurrentArea())) {
        				    			x = (npcX + playerX + 640) / 32;
	        				    		y = (npcY + playerY + 480) / 32;
        				    		}
        				    		if (x == index1 && y == index2) {
        				    			npc.setSelected(true);
        				    			//return;
        				    		} else {
        				    			npc.setSelected(false);
        				    		}
        						}
        					}
        				}
	        		}
	        	}
    		}
    	}
    }


    /**
     * keyPressed
     * @param keyEvent The key that gets pressed.
     */
    public void keyPressed(KeyEvent keyEvent) {
    	GameEntity player = renderer.getGame().getCharacter();

		// movement
    	if (keyEvent.getKeyCode() == KeyEvent.VK_UP) {
    		if (player.getMoving() == GameEntity.STOPPED) {
    			// collision detection
	    		if (player.collisionDetection(GameEntity.NORTH) == 0) {
	    			// move north
	        		player.setMoving(GameEntity.NORTH);
	    		} else if (player.collisionDetection(GameEntity.NORTH) == 3) {
	    			// move north and load new area
	        		player.setMoving(GameEntity.NORTH);
	    			enteringArea = true;
	    		}
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_DOWN) {
    		if (player.getMoving() == GameEntity.STOPPED) {
	    		// collision detection
	    		if (player.collisionDetection(GameEntity.SOUTH) == 0) {
	    			// move south
	        		player.setMoving(GameEntity.SOUTH);
	    		} else if (player.collisionDetection(GameEntity.SOUTH) == 3) {
	    			// move south and load new area
	        		player.setMoving(GameEntity.SOUTH);
	    			enteringArea = true;
	    		}
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_LEFT) {
    		if (player.getMoving() == GameEntity.STOPPED) {
	    		// collision detection
	    		if (player.collisionDetection(GameEntity.WEST) == 0) {
	    			// move west
	        		player.setMoving(GameEntity.WEST);
	    		} else if (player.collisionDetection(GameEntity.WEST) == 3) {
	    			// move west and load new area
	        		player.setMoving(GameEntity.WEST);
	    			enteringArea = true;
	    		}
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_RIGHT) {
    		if (player.getMoving() == GameEntity.STOPPED) {
	    		// collision detection
	    		if (player.collisionDetection(GameEntity.EAST) == 0) {
	    			// move east
	        		player.setMoving(GameEntity.EAST);
	    		} else if (player.collisionDetection(GameEntity.EAST) == 3) {
	    			// move east and load new area
	        		player.setMoving(GameEntity.EAST);
	    			enteringArea = true;
	    		}
    		}
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_A) {
    		for (int index = 0; index < renderer.getGame().getWorld().getNPCs().size(); index++) {
	    		GameEntity npc = (GameEntity) renderer.getGame().getWorld().getNPCs().get(index);
	    		if (npc.isSelected()) {
	    			if (renderer.getGame().getCharacter().adjacentToPlayer()) {
	    				renderer.getGame().getCharacter().attack(GameEntity.DAMAGE, npc);
	    			}
	    		}
    		}
    		// TODO: maybe have a direction facing the enemy
    		// TODO: maybe use a mouse for attacking
    	} else if (keyEvent.getKeyCode() == KeyEvent.VK_ESCAPE) {
    		// TODO: menu popup
    		renderer.setGameRunning(false);
    	}

    	// TODO: delete
    	/*System.out.println(player.getCurrentArea().getName()
    					 + ": "
    					 + player.getCurrentLocation().getX() + ", "
    					 + player.getCurrentLocation().getY());*/
    }


    /**
     * loadImages
     */
    public void loadImages() {
    	Area centerArea = renderer.getGame().getCharacter().getCurrentArea();
    	Area topArea = renderer.getGame().getWorld().getArea(centerArea.getAbove());
    	Area bottomArea = renderer.getGame().getWorld().getArea(centerArea.getBelow());

    	topLeft = (new ImageIcon("Images/World/" + topArea.getLeft() + ".jpg")).getImage();
    	top = (new ImageIcon("Images/World/" + topArea.getName() + ".jpg")).getImage();
    	topRight= (new ImageIcon("Images/World/" + topArea.getRight() + ".jpg")).getImage();
    	left = (new ImageIcon("Images/World/" + centerArea.getLeft() + ".jpg")).getImage();
    	center = (new ImageIcon("Images/World/" + centerArea.getName() + ".jpg")).getImage();
    	right = (new ImageIcon("Images/World/" + centerArea.getRight() + ".jpg")).getImage();
    	bottomLeft = (new ImageIcon("Images/World/" + bottomArea.getLeft() + ".jpg")).getImage();
    	bottom= (new ImageIcon("Images/World/" + bottomArea.getName() + ".jpg")).getImage();
    	bottomRight = (new ImageIcon("Images/World/" + bottomArea.getRight() + ".jpg")).getImage();
    }


    /**
     * removeColor
     * @param image The given image.
     * @param color The color to be removed from the image.
     * @return Returns an image with a color removed from it.
     */
    public BufferedImage removeColor(BufferedImage image, Color color) {
    	BufferedImage newImage = new BufferedImage(image.getWidth(), image.getHeight(), image.getType());

    	newImage.createGraphics().drawImage((Image) image, 0, 0, null);
    	for (int index1 = 0; index1 < 32; index1++) {
    		for (int index2 = 0; index2 < 32; index2++) {
    			if ((image.getRGB(index1, index2) | 0xFF000000) == color.getRGB()) {
    				newImage.setRGB(index1, index2, color.getRGB() & 0x00FFFFFF);
    			}
    		}
    	}

    	return newImage;
    }


    /**
     * timeChanges
     * @param delta The time it took from the last game loop to the current one.
     */
    public void timeChanges(long delta) {
    	GameEntity player = renderer.getGame().getCharacter();

    	// player movement
    	if (player.getMoving() != GameEntity.STOPPED && player.getMovement() < 32) {
    		// set previous location
    		if (player.getMovement() == 0) {
    			player.setPreviousLocation(player.getCurrentLocation().getX(),
    									   player.getCurrentLocation().getY());
    			player.setPreviousArea(player.getCurrentArea());
			}

    		// move player
    		timeCounter += delta * .001;
    		if (timeCounter >= 1.0/128.0) {
				if (renderer.getGame().getCharacter().move(player.getMoving())) {
					loadImages();
				}
	    		timeCounter = 0;
	    		player.setMovement(player.getMovement() + player.getAttributes().getMovementRate());
    		}
    	} else {
    		if (enteringArea) {
    			player.enteringNewArea(player.getMoving());
    			loadImages();
    		}
    		player.checkSameLocation();
    		player.setMoving(GameEntity.STOPPED);
    		player.setMovement(0);
    		enteringArea = false;
    	}

    	// creature movement - currently every 5 seconds for all creatures and it's random
    	// also checks if a creature's going into the same spot
    	creatureTimeCounter += delta * .001;
    	if (creatureTimeCounter >= 5) {
	    	for (int index1 = 0; index1 < renderer.getGame().getWorld().getNPCs().size(); index1++) {
	    		GameEntity npc = (GameEntity) renderer.getGame().getWorld().getNPCs().get(index1);
	    		Random randomNumberGenerator = new Random();
	    		double randomDirection = randomNumberGenerator.nextDouble() * 5;
	    		if (npc.collisionDetection((int) randomDirection) == 0) {
	    			npc.setMoving((int) randomDirection);
	    		}
	    	}
	    	creatureTimeCounter = 0;
    	}
    	for (int index = 0; index < renderer.getGame().getWorld().getNPCs().size(); index++) {
    		GameEntity npc = (GameEntity) renderer.getGame().getWorld().getNPCs().get(index);

    		if (npc.getMoving() != GameEntity.STOPPED && npc.getMovement() < 32) {
    			if (npc.getMovement() == 0) {
    				npc.setPreviousLocation(npc.getCurrentLocation().getX(),
    										npc.getCurrentLocation().getY());
    				npc.setPreviousArea(npc.getCurrentArea());
    			}
    			creatureMovementCounter += delta * .001;
    			if ((creatureMovementCounter) >= 1.0/128.0) {
    				npc.move(npc.getMoving());
    				creatureMovementCounter = 0;
    	    		npc.setMovement(npc.getMovement() + npc.getAttributes().getMovementRate());
    			}
    		} else {
    			npc.checkSameLocation();
				npc.setMoving(GameEntity.STOPPED);
				npc.setMovement(0);
			}
    	}
    }


    /**
     * render
     * @param graphics The graphics display.
     */
    public void render(Graphics2D graphics) {
    	int playerX = renderer.getGame().getCharacter().getCurrentLocation().getX();
    	int playerY = renderer.getGame().getCharacter().getCurrentLocation().getY();

    	// draw the area based on the player's location
    	graphics.drawImage(topLeft, (playerX - 640) - 16, playerY - 480, null);
    	graphics.drawImage(top, playerX - 16, playerY - 480, null);
    	graphics.drawImage(topRight, (playerX + 640) - 16, playerY - 480, null);
    	graphics.drawImage(left, (playerX - 640) - 16, playerY, null);
    	graphics.drawImage(center, playerX - 16, playerY, null);
    	graphics.drawImage(right, (playerX + 640) - 16, playerY, null);
    	graphics.drawImage(bottomLeft, (playerX - 640) - 16, playerY + 480, null);
    	graphics.drawImage(bottom, playerX - 16, playerY + 480, null);
    	graphics.drawImage(bottomRight, (playerX + 640) - 16, playerY + 480, null);

    	// draw the NPCs
    	for (int index = 0; index < renderer.getGame().getWorld().getNPCs().size(); index++) {
    		GameEntity npc = (GameEntity) renderer.getGame().getWorld().getNPCs().get(index);
    		Area centerArea = renderer.getGame().getCharacter().getCurrentArea();
    		Area topArea = renderer.getGame().getWorld().getArea(centerArea.getAbove());
    		Area bottomArea = renderer.getGame().getWorld().getArea(centerArea.getBelow());
    		BufferedImage npcImage = new BufferedImage(32, 32, BufferedImage.TYPE_INT_ARGB);
    		npcImage.createGraphics().drawImage((Image) npc.getSprite(), 0, 0, null);
    		graphics.setColor(Color.YELLOW);
    		if (npc.getCurrentArea().equals(renderer.getGame().getWorld().getArea(topArea.getLeft()))) {
    			graphics.drawImage(removeColor(npcImage, Color.WHITE),
    							   null,
								   playerX + npc.getCurrentLocation().getX() - 640 - 16,
								   playerY + npc.getCurrentLocation().getY() - 480);
    			if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() - 640 - 16,
    								  playerY + npc.getCurrentLocation().getY() - 480,
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(topArea)) {
    			graphics.drawImage(removeColor(npcImage, Color.WHITE),
								   null,
								   playerX + npc.getCurrentLocation().getX() - 16,
								   playerY + npc.getCurrentLocation().getY() - 480);
    			if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() - 16,
    								  playerY + npc.getCurrentLocation().getY() - 480,
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(renderer.getGame().getWorld().getArea(topArea.getRight()))) {
    			graphics.drawImage(removeColor(npcImage, Color.WHITE),
								   null,
								   playerX + npc.getCurrentLocation().getX() + 640 - 16,
								   playerY + npc.getCurrentLocation().getY() - 480);
    			if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() + 640 - 16,
    								  playerY + npc.getCurrentLocation().getY() - 480,
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(renderer.getGame().getWorld().getArea(centerArea.getLeft()))) {
    			graphics.drawImage(removeColor(npcImage, Color.WHITE),
    							   null,
								   playerX + npc.getCurrentLocation().getX() - 640 - 16,
								   playerY + npc.getCurrentLocation().getY());
    			if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() - 640 - 16,
    								  playerY + npc.getCurrentLocation().getY(),
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(centerArea)) {
	    		graphics.drawImage(removeColor(npcImage, Color.WHITE),
	    						   null,
	    						   playerX + npc.getCurrentLocation().getX() - 16,
	    						   playerY + npc.getCurrentLocation().getY());
	    		if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() - 16,
    								  playerY + npc.getCurrentLocation().getY(),
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(renderer.getGame().getWorld().getArea(centerArea.getRight()))) {
	    		graphics.drawImage(removeColor(npcImage, Color.WHITE),
	    						   null,
								   playerX + npc.getCurrentLocation().getX() + 640 - 16,
								   playerY + npc.getCurrentLocation().getY());
	    		if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() + 640 - 16,
    								  playerY + npc.getCurrentLocation().getY(),
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(renderer.getGame().getWorld().getArea(bottomArea.getLeft()))) {
	    		graphics.drawImage(removeColor(npcImage, Color.WHITE),
	    						   null,
								   playerX + npc.getCurrentLocation().getX() - 640 - 16,
								   playerY + npc.getCurrentLocation().getY() + 480);
	    		if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() - 640 - 16,
    								  playerY + npc.getCurrentLocation().getY() + 480,
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(bottomArea)) {
	    		graphics.drawImage(removeColor(npcImage, Color.WHITE),
	    						   null,
								   playerX + npc.getCurrentLocation().getX() - 16,
								   playerY + npc.getCurrentLocation().getY() + 480);
	    		if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() - 16,
    								  playerY + npc.getCurrentLocation().getY() + 480,
    								  32, 32);
    			}
    		} else if (npc.getCurrentArea().equals(renderer.getGame().getWorld().getArea(bottomArea.getRight()))) {
    			graphics.drawImage(removeColor(npcImage, Color.WHITE),
								   null,
								   playerX + npc.getCurrentLocation().getX() + 640 - 16,
								   playerY + npc.getCurrentLocation().getY() + 480);
    			if (npc.isSelected()) {
    				graphics.drawRect(playerX + npc.getCurrentLocation().getX() + 640 - 16,
    								  playerY + npc.getCurrentLocation().getY() + 480,
    								  32, 32);
    			}
    		}
    	}

    	// draw the player
		BufferedImage character = new BufferedImage(32, 32, BufferedImage.TYPE_INT_ARGB);
    	character.createGraphics().drawImage((Image) renderer.getGame().getCharacter().getSprite(), 0, 0, null);
    	graphics.drawImage(removeColor(character, Color.WHITE), null, 304, 224);

    	// draw the transparent GUI
    	BufferedImage quickKeyTransparency = new BufferedImage(32, 32, BufferedImage.TYPE_INT_ARGB);
    	quickKeyTransparency.createGraphics().drawImage(renderer.getGame().getCharacter().getSprite(), 0, 0, null);
        AlphaComposite alphaComposite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
        graphics.setComposite(alphaComposite);
    	graphics.drawImage(quickKeyTransparency, null, 8, 440);
    }
}